<?php
/**
 * This is the BsAdapter prototype.
 * 
 * Every adapter class have to be inherited from this prototype.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Sebastian Ulbricht <sebastian.ulbricht@dragon-design.hk>
 * @author     Robert Vogel <vogel@hallowelt.biz>
 * @version    0.1.0
 * @version    $Id: Adapter.class.php 6438 2012-09-10 11:04:04Z smuggli $
 * @package    Bluespice_Core
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * the BsAdapter prototype class
 * @package BlueSpice_Core
 * @subpackage Core
 */
abstract class BsAdapter {
	/**
	 * holds the name and the filepath of every registered adapter as an assoziative array
	 * @var array
	 * @access protected
	 */
	protected static $prRegisteredAdapters = array();
	/**
	 * holds the requested URI after the first time, the method getRequestURI was running
	 * @var string
	 */
	protected static $prRequestUri = null;
	/**
	 * a state flag if the requested URL is encodet
	 * @var bool
	 */
	protected static $prUrlIsEncoded = false;

	/**
	 * Use this method to register a new adapter.
	 * 
	 * The adapter class has to be inherited from the BsAdapter class and the classname
	 * has to match the form "BsAdapter<the given name>". So if the name of the adapter
	 * is Test, the classname would be BsAdapterTest.
	 * 
	 * @param string $sName the adapter name
	 * @param string $sPath the path to the adapters classfile
	 */
	public static function registerAdapter($sName, $sPath) {
		self::$prRegisteredAdapters[$sName] = $sPath;
	}

	/**
	 * This method loads the adapter with the given name and returns an instance of it.
	 * 
	 * @param string $sName the adapter name
	 * @return object a instance of the adapter class
	 */
	public static function loadAdapter($sName) {
		wfProfileIn( 'BS::'.__METHOD__ );
		if(!isset(self::$prRegisteredAdapters[$sName])) {
			throw new Exception('GIVEN_ADAPTER_NOT_FOUND', 1000);
		}
		require(self::$prRegisteredAdapters[$sName].DS.'Adapter.class.php');
		$class = 'BsAdapter'.$sName;
		$instance = new $class();
		wfProfileOut( 'BS::'.__METHOD__ );
		return $instance;
	}

	/**
	 * Determines the request URI for Apache and IIS
	 * 
	 * @param bool $getUrlEncoded set to true to get URI url encoded
	 * @return string the requested URI
	 */
	public static function getRequestURI($getUrlEncoded = false) {
		wfProfileIn( 'BS::'.__METHOD__ );
		if (self::$prRequestUri === null) {
			$requestUri = '';
			if (isset($_SERVER['HTTP_X_REWRITE_URL'])) { // check this first so IIS will catch
				$requestUri = $_SERVER['HTTP_X_REWRITE_URL'];
			} elseif (isset($_SERVER['REQUEST_URI'])) {
				$requestUri = $_SERVER['REQUEST_URI'];
			} elseif (isset($_SERVER['ORIG_PATH_INFO'])) { // IIS 5.0, PHP as CGI
				$requestUri = $_SERVER['ORIG_PATH_INFO'];
				if (!empty($_SERVER['QUERY_STRING'])) {
					$requestUri .= '?' . $_SERVER['QUERY_STRING'];
				}
			}
			self::$prRequestUri = $requestUri;
			self::$prUrlIsEncoded = (urldecode(self::$prRequestUri) != self::$prRequestUri);
		}
		if($getUrlEncoded) {
			return (self::$prUrlIsEncoded ? self::$prRequestUri : urlencode(self::$prRequestUri));
		}
		wfProfileOut( 'BS::'.__METHOD__ );
		return (self::$prUrlIsEncoded ? urldecode(self::$prRequestUri) : self::$prRequestUri);
	}

	/**
	 * This method spends a possibility to get the name of variables in an adapter specific form.
	 * @param string $sName the name of the variable
	 * @param bool $bSearchGlobals true to search for the variable in global scope
	 * @return string the variable name in adapter specific form (with pre-/postfix or something similar)
	 */
	public abstract function getVarName($name, $global = false);

	/**
	 * setup the adapter
	 */
	public abstract function doInitialise();
}