<?php

/**
 * Decorator pattern to use with bluespice core.
 * Let your classes extend BsDecorator and call
 * parent::__construct( $instanceToDecorate ) to inject any object into ist.
 * Now any call to your object that addresses methods or properties that can not
 * be found will be fetched from $instanceToDecorate, kind of fallback.
 * Known drawbacks
 * - is_a($decoratedInstance, get_class($decoratedInstance)) === false
 * - functions of $decoratedInstance that have references in their signature
 *   will not work with PHP >= 5.3
 */
abstract class BsDecorator {

	private $decoratedInstance;

	public function __construct( $instanceToDecorate ) {
		$this->decoratedInstance = $instanceToDecorate;
	}

	/**
	 * called if property $this->$name is accessed but unset in extending instance
	 * if property does not exist but $decoratedInstance has it's own magic getter
	 * this one will be called as well.
	 * @param string $name
	 * @return mixed  
	 */
	public function __get( $name ) {
		if ( isset( $this->decoratedInstance->$name ) )
			return $this->decoratedInstance->$name;
		$callback = array( $this->decoratedInstance, '__get' );
		return ( is_callable( $callback ) )
			? call_user_func_array( $callback, array( $name ) )
			: null;
		// return null; equivalent to return;
		// accessing $anyObject->inaccessibleProperty returns null anyway!
	}
	
	/**
	 * Intentionally omitted. $this->$key = $val should create a property in the
	 * extending instance, not the instance to be decorated
	 */
	//public function __set($key, $val)

	/*
	 * called if method $this->$name( $arguments ) is inaccessible in extending
	 * instance
	 * @param string $name name of called method
	 * @param array $arguments arguments to called method
	 * Note (php.net): None of the arguments can be passed by reference
	 */
	public function __call( $name, $arguments ) {
		$callback = array( $this->decoratedInstance, $name );
		return ( is_callable( $callback ) )
			? call_user_func_array( $callback, $arguments )
			: null;
	}
	
	/**
	 * Intentionally omitted. __callStatic came with PHP 5.3. Our precondition
	 * is to work with PHP 5.2 as well
	 */
	//public function __callStatic( $name, $arguments )

}