<?php
/**
 * Base class for ExtendedSearch
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: ExtendedSearchBase.class.php 7685 2012-12-11 19:05:29Z smuggli $
 * @package    BlueSpice_Core
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Base class for ExtendedSearch
 * @package BlueSpice_Core
 * @subpackage ExtendedSearch
 */
abstract class BsExtendedSearchBase {

	/**
	 * Array of instances
	 *
	 * Entries of type 'module' => &moduleInstance
	 * @var array
	 */
	protected static $aoInstances = array( );
	/**
	 * Instance of object that holds the current search request.
	 * @var BsSearchRequest
	 */
	protected static $oSearchRequestCore = null;
	/**
	 * Search initialization state.
	 * @var bool True if already initialized.
	 */
	protected static $bInitialized = false;
	/**
	 * Reference to current search service.
	 * @var BsSearchService Current search service object. 
	 */
	protected static $oSolrSearchService = null;

	/**
	 * Register a search data source.
	 * @param BsExtendedSearchBase $oInstance Search data source to be registered.
	 */
	public static function registerInstance( &$oInstance ) {
		self::initStatic();
		if ( !empty( self::$aoInstances[$oInstance->sFrameworkName] ) ) // !empty returns true if array key is set and value is not null
			throw new BsException( 'Can not register Datasource '.$oInstance->sFrameworkName.' twice!' );
		self::$aoInstances[$oInstance->sFrameworkName] = $oInstance;
	}

	/**
	 * Fetches a data source
	 * @param string $sName
	 * @return BsExtendedSearchBase
	 */
	public static function &getInstance( $sName ) {
		if ( empty( self::$aoInstances[$sName] ) ) { // empty returns true if array key is not set or value is null
			throw new BsException( "Instance $sName unknown in ".__METHOD__ );
		}
		return self::$aoInstances[$sName];
	}

	/**
	 * Fetches the current search request obejct.
	 * @param string $sOrigin Specifiy which search request to process
	 * @return BsSearchRequest
	 */
	public static function &getSearchRequestCore( $sOrigin = 'search_form_body' ) {
		if ( self::$oSearchRequestCore === null )
			self::$oSearchRequestCore = new BsSearchRequest( $sOrigin );
		return self::$oSearchRequestCore;
	}

	/**
	 * Fetches instance of solr search service.
	 * @return BsSearchService
	 */
	public static function &getSolrSearchService() {
		if ( self::$oSolrSearchService === null ) {
			$aParams = array( // todo: add customerId and Solr-UTF8 here and in SearchService.class.php
				'protocol' => BsConfig::get( 'Core::Search::SolrProtocol' ),
				'host'     => BsConfig::get( 'Core::Search::SolrHost' ),
				'port'     => BsConfig::get( 'Core::Search::SolrPort' ),
				'path'     => BsConfig::get( 'Core::Search::SolrPath' ),
				'pingTime' => BsConfig::get( 'Core::Search::SolrPingTime' )
			);
			self::$oSolrSearchService = BsSearchService::createInstance( 'Core', 'solr', $aParams );
		}
		return self::$oSolrSearchService;
	}

	/**
	 * Checks if curl-extension is activated
	 * @return boolean
	 */
	public static function isCurlActivated() {
		return in_array ( 'curl', get_loaded_extensions() ) ? true : false;
	}

	/**
	 * Initialize all variables from settings.
	 * @return void Only used when already initialized.
	 */
	public static function initStatic() {
		if ( BsExtendedSearchBase::$bInitialized === true ) return;

		BsConfig::registerVar( 'Core::Search::FormMethod',        'get',                     BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-formmethod' );
		BsConfig::registerVar( 'Core::Search::HighlightSnippets', '3',                       BsConfig::TYPE_INT|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-highlightsnippets', 'int' );
		BsConfig::registerVar( 'Core::Search::IndexFileTypes',    'doc, pdf, ppt, xls, txt', BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-indexfiletypes' );
		BsConfig::registerVar( 'Core::Search::LogUsers',          true,                      BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-logusers', 'toggle' );
		BsConfig::registerVar( 'Core::Search::Logging',           true,                      BsConfig::TYPE_BOOL|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-logging', 'toggle' );
		BsConfig::registerVar( 'Core::Search::MaxDocSizeMB',      10,                        BsConfig::TYPE_INT );
		BsConfig::registerVar( 'Core::Search::SelectorHeight',    10,                        BsConfig::TYPE_INT );
		BsConfig::registerVar( 'Core::Search::SelectorWidth',     '200px',                   BsConfig::TYPE_STRING );
		BsConfig::registerVar( 'Core::Search::SolrProtocol',      'http',                    BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-solrprotocol' );
		BsConfig::registerVar( 'Core::Search::SolrHost',          '127.0.0.1',               BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-solrhost' );
		BsConfig::registerVar( 'Core::Search::SolrPort',          '8080',                    BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-solrport' );
		BsConfig::registerVar( 'Core::Search::SolrPath',          '/solr3.5',                BsConfig::TYPE_STRING|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-solrpath' );
		BsConfig::registerVar( 'Core::Search::SolrPingTime',      3,                         BsConfig::TYPE_INT|BsConfig::LEVEL_PUBLIC, 'bs-extendedsearch-pref-solrpingtime', 'int' );
		BsExtendedSearchBase::$bInitialized = true;
	}

	/**
	 * Triggers search for all instances
	 * @param array $aMonitor Set of options.
	 * @return ViewBaseElement Search result view
	 */
	public static function searchAllModules( &$aMonitor ) {
		$oSearchRequestCore = & self::getSearchRequestCore();
		$aResultViews = array( );
		foreach ( self::$aoInstances as $name => $instance ) {
			$vItem = $instance->search( $oSearchRequestCore, $aMonitor );

			if ( $vItem instanceof ViewBaseElement ) {
				$aResultViews[$name] = $vItem;
			}
			else {
				throw new BsException( 'Search module returned no view!' );
			}
		}
		return $aResultViews;
	}

	/**
	 * Triggers re-building of index
	 * May be called via ajax or commandline;
	 * @param array $aTypes optional, e.g. array('wiki' => true, 'repo' => false, 'linked' => false, 'external'=> false)
	 * @param array $aFileTypes optional, e.g. array('doc' => true, 'pdf' => true, 'ppt' => true, 'xls' => true, 'txt' => false, 'sh' => false, 'sql' => false, 'html' => false)
	 * @param string $sLimit optional Syntax is "[0-9]+,[0-9]+". Global limit for SQL statements. If set it will be appended in SQL-Strings with ." limit $limit".
	 * @param integer iTimeLimit timelimit.
	 * @return string
	 */
	public static function buildIndexForAllModules( $aTypes = null, $aFileTypes = null, $sLimit = null, $iTimeLimit = null ) {
		self::initStatic();
		try {
			self::getSolrSearchService(); // may throw BsException('server-not-available');
		}
		catch ( BsException $e ) {
			wfDebugLog( 'ExtendedSearch', 'onArticleDeleteComplete: '.$e->getMessage() );
		}

		$oBuildIndexMainControl = self::getBuildIndexMainControl();
		$res = $oBuildIndexMainControl->buildIndex( self::$aoInstances, $sLimit, $aTypes, $iTimeLimit );
		return $res;
	}

	/**
	 * Creates a new index build controller.
	 * @return BsBuildIndexMainControl 
	 */
	public static function getBuildIndexMainControl() {
		// CR MRG (27.06.11 16:45): Cachen?
		return new BsBuildIndexMainControl( self::getSolrSearchService() );
	}

	/*
	 * BsExtendedSearch has to be inherited by each ExtendedSearch-extension of a blue spice module.
	 * The instances are registered in the static class content.
	 * Below the core functionality is implemented non-static and has to be instanciated
	 */

	/**
	 * Instance of current search service.
	 * @var BsSearchService
	 */
	protected $oSearchService = null;
	/**
	 * Instance of current search index.
	 * @var BsSearchIndex
	 */
	protected $oSearchIndex = null;
	/**
	 * Instance of current search request.
	 * @var BsSearchRequest
	 */
	protected $oSearchRequest = null;
	//protected $oBuildIndex = null;
	/**
	 * Name of instance or framework.
	 * @var string
	 */
	protected $sFrameworkName;
	/**
	 * Name of extension.
	 * @var string
	 */
	protected $sExtensionName;

	/*
	 * todo: $searchRequest typsicher machen
	 */
	/**
	 * This abstract function searches the index for a given search term
	 * @param BsSearchRequest $oSearchRequestCore Current search request
	 * @param array &$aMonitor Set of options.
	 */
	abstract public function &search( BsSearchRequest $oSearchRequestCore, &$aMonitor );

	/**
	 * This abstract function defines a standard function for building an index.
	 * @param BsBuildIndexMainControl $oMainControl
	 * @param int $iLimit {Limit}{Offset}
	 * @param array $aFiletypes List of filetyes that need to be indexed.
	 * @param array $aTypes List of overall types, e.g. wiki or files
	 * @param int $iTimeLimit Maximum execution time per document in seconds.
	 */
	abstract public function buildIndex( $oMainControl, $iLimit, $aFiletypes, $aTypes, $iTimeLimit );

}