<?php
/**
 * Index builder control for ExtendedSearch
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: BuildIndexMainControl.class.php 7747 2012-12-17 15:16:47Z smuggli $
 * @package    BlueSpice_Core
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
// todo: $dbr->freeResult um Speicher freizugeben
/**
 * Index builder control for ExtendedSearch
 * @package BlueSpice_Core
 * @subpackage ExtendedSearch
 */
class BsBuildIndexMainControl {

	/**
	 * Instance of current search service
	 * @var BsSearchService
	 */
	protected $oSearchService;
	/**
	 * Path to file where index progress is stored.
	 * @var string Path
	 */
	protected $sFilePathIndexProgTxt;
	/**
	 * Path to log file.
	 * @var string Path. 
	 */
	protected $sFilePathLogFile;
	/**
	 * Path to lock file.
	 * @var string Path. 
	 */
	protected $sFilePathLockFile;
	/**
	 * Maximum size for documents to be indexed.
	 * @var int File size in bytes.
	 */
	protected $iMaxDocSize;
	/**
	 * Unique ID that identifies a certain Wiki
	 * @var string Unique ID
	 */
	protected $sCustomerId;
	/**
	 * Indicates whether builder was called from console
	 * @var bool True if called in cli mode.
	 */
	protected $bCommandLineMode;
	/**
	 * Number of documents in index.
	 * @var int
	 */
	protected $iDocsInIndex = 0;
	/**
	 * Array of file types that should be indexed
	 * @var array
	 */
	protected $aFileTypes = array();

	// make your non-object-variables all readonly :-)
	/**
	 * Magic getter method.
	 * @param string $sName Name of the property to get.
	 * @return mixed Value of property to get.
	 */
	public function __get( $sName ) {
		return ( isset( $this->$sName ) ) ? $this->$sName : null;
	}

	/**
	 * Constructor for BsBuildIndexMainControl
	 * @param BsSearchService $oSearchService Search service to use
	 */
	public function __construct( $oSearchService ) {
		$this->oSearchService = $oSearchService;

		$this->sFilePathIndexProgTxt = BSDATADIR.DS.'index_prog.txt';
		$this->sFilePathLogFile = BSDATADIR.DS.'ExtendedSearch.log';
		$this->sFilePathLockFile = BSDATADIR.DS.'ExtendedSearch.lock';

		$this->iMaxDocSize = ( BsConfig::get( 'Core::Search::MaxDocSizeMB', 10 ) * 1024 * 1024 );
		$this->sCustomerId = BsConfig::get( 'Core::Search::CustomerID' );

		//Possible values of PHP_SAPI (not all): apache, cgi (until PHP 5.3), cgi-fcgi, cli
		$this->bCommandLineMode = ( PHP_SAPI === 'cli' );
	}

	/**
	 * Main method to add documents to search index
	 * @param Apache_Solr_Document $oSolrDocument to send to search service and add to the index
	 * @param string $sMode In case of an error trigger writeLog with this mode (i18n'ned string)
	 * @param string $sMessageOnError In case of an error trigger writeLog with this message
	 * @param int $iTryagain Defaults to 0. In case of exception method is called recursively with tryagain incremented automatically
	 */
	public function addDocument( $oSolrDocument, $sMode, $sMessageOnError, $iTryagain = 0 ) {
		// Add the document to the index
		try {
			$this->oSearchService->addDocument( $oSolrDocument );
			$this->iDocsInIndex++;
			wfRunHooks( 'BS:ExtendedSearch:AddDocumentLog', array( $oSolrDocument ) );
		}
		catch ( Exception $ex ) {
			if ( $iTryagain >= 2 ) { // todo: make no of retries configurable
				$this->writeLog( $sMode, $sMessageOnError, 0 );
				// todo: count documents, that have not been able to index and output this statistic
			}
			else {
				// maybe solr has had too much to do so it was not always able to answer
				sleep( 3 ); // todo: make timespan to wait configurable
				$this->addDocument( $oSolrDocument, $sMode, $sMessageOnError, ++$iTryagain );
			}
		}
	}

	/**
	 * Starts building of the index for every search-source.
	 * External is assigned to core, other modules
	 * like wiki or social network are related to bluespice extensions. The latter
	 * are registered to BsExtendedSearchBase::$aoInstances and are injected as
	 * param $aExtendedSearchBases
	 * They itself care about whether to index or not based on their own settings, but
	 * it is possible to override these with arguments from commandline
	 * @param array $aExtendedSearchBases
	 * @param array $aTypes
	 * @param array $aLimit
	 * @return string
	 */
	public function buildIndex( $aExtendedSearchBases, $aLimit, $aTypes, $iTimeLimit ) {
		ignore_user_abort( true ); // defaults to false; script execution is terminated if client disconnects
		// todo: first clean all output buffers, second set Content-Encoding to none in headers
		while ( @ob_end_clean() );
		if ( !$this->bCommandLineMode ) header( "Content-Encoding: none" );
		// todo: test
		flush(); // http://de3.php.net/manual/en/function.flush.php
		// todo: what if zlib.output_compression = On in php.ini

		$this->logFile( 'delete' );

		/* $limit is null OR string with syntax "{start},{range}"
		 * if not set it defaults to false
		 *     then the existence of $_GET['start'] and $_GET['range'] is checked
		 *     in case of presence a limit is set with these parameters casted as integers */
		if ( $aLimit === null && !$this->bCommandLineMode ) {
			$limitStart = BsCore::getParam( 'start', false, BsPARAM::GET|BsPARAMTYPE::INT );
			$limitRange = BsCore::getParam( 'range', false, BsPARAM::GET|BsPARAMTYPE::INT );
			if ( $limitStart !== false && $limitRange !== false )
					$aLimit = $limitStart.','.$limitRange;
		}
		if ( $aLimit !== null ) {
			$aLimit = explode( ',', $aLimit );
			if ( count( $aLimit ) != 2 )
					throw new BsException( 'Invalid limit in '.__FILE__.', method'.__METHOD__ );
			unset( $aLimit );
		}

		if ( $this->aFileTypes === null || !is_array( $this->aFileTypes ) || empty( $this->aFileTypes ) ) {
			$this->setFileTypes();
		}

		if ( $iTimeLimit === null ) {
			if ( PHP_SAPI === 'cli' ) { 
				$iTimeLimit = 120; 
			} else {
				$iTimeLimit = ini_get( 'max_execution_time' );
			}
		}

		$res = '';
		$this->iDocsInIndex = 0;
		foreach ( $aExtendedSearchBases as $name => $instance ) {
			try {
				if ( file_exists( $this->sFilePathLockFile ) ) {
					 return $res = wfMessage( 'bs-extendedsearch-indexinginprogress' )->plain();
				}
				$this->logFile( 'createLock' );
				$instance->buildIndex( $this, $aLimit, $this->aFileTypes, $aTypes, $iTimeLimit );
			}
			catch ( BsException $e ) {
				$this->logFile( 'deleteLock' );
				$res .= "Instance $name returned following BsException in procedure buildIndex(): {$e->getMessage()}";
			}
		}

		// rebulding Spellchecker index
		try {
			$this->oSearchService->getSpellcheck( 'dummy', array( 'spellcheck.build' => 'true' ), true );
			$res .= wfMessage( 'bs-extendedsearch-spellindexcreated' )->plain();
			$this->writeLog( wfMessage( 'bs-extendedsearch-indexingcomponent' )->plain(), wfMessage( 'bs-extendedsearch-spell' )->plain(), 100 );
		}
		catch ( BsException $e ) {
			$this->writeLog( 'Indexing component', "Error while creating spellcheck index: {$e->getMessage()}" );
		}

		$this->oSearchService->commit( true, true, true, 60 );

		$this->writeLog( wfMessage( 'bs-extendedsearch-finished' )->plain() . ' ' . wfMessage( 'bs-extendedsearch-docs-in-index' )->plain(), $this->iDocsInIndex, 100 );
		$this->writeLog( 'unlink' );
		$this->logFile( 'deleteLock' );
		return $res;
	}

	/**
	 * Sets the file types that should be indexed
	 */
	public function setFileTypes() {
		$tempFileTypes = BsConfig::get( 'Core::Search::IndexFileTypes' ); // 'doc, pdf, ppt, xls, txt'
		$tempFileTypes = str_replace( array( ' ', ';' ), array( '', ',' ), $tempFileTypes );
		$tempFileTypes = explode( ',', $tempFileTypes );
		foreach ( $tempFileTypes as $value )
			$this->aFileTypes[$value] = true;
		unset( $tempFileTypes );
		return $this->aFileTypes;
	}

	/**
	 * Central routine for converting document attributes to Apache_Solr_Document,
	 * that can be transferred to the Solr-Server and be stored in the index.
	 * @param string $sOverallType Of type 'wiki', 'repo', 'linked', 'external' or 'special-linked' (sharepoint)
	 * @param string $sType Of type 'wiki', 'doc', 'ppt', 'xls', 'pdf', 'txt', 'html', 'sql', 'sh', ...
	 * @param string $sTitle Title or filename
	 * @param string $sText The body of the wiki-page or the document
	 * @param int $iID Id is -1 in case it's no wiki-article
	 * @param int $iNamespace Namespace of document, is 999 in case it's no wiki-article
	 * @param mixed $vPath Path to document if external (not in wiki). Might be empty or null
	 * @param unknown $iTimestamp Timestamp
	 * @param array $aCategories Categories
	 * @param array $aEditors Editors
	 * @return Apache_Solr_Document
	 */
	public function makeDocument( $sOverallType, $sType, $sTitle, $sText, $iID, $iNamespace, $vPath, $iTimestamp, $aCategories = array(), $aEditors = array() ) {
		$doc = new Apache_Solr_Document();

		$doc->wiki         = $this->sCustomerId;
		$doc->overall_type = $sOverallType;
		$doc->type         = $sType;
		$doc->title        = $sTitle;
		$doc->text         = $sText;
		$doc->hwid         = $iID;
		$doc->namespace    = $iNamespace;
		$doc->path         = $vPath;
		$doc->cat          = $aCategories;
		$doc->editor       = $aEditors;
		$doc->uid          = $this->getUniqueId( $iID, $vPath );

		// Date must be of the format 1995-12-31T23:59:59Z
		// If makeDocument is trigged by onArticleSaveComplete for example,
		// the timestamp is wfTimestamp and in this case you don't have to change it.
		// Only in case the timestamp is from the database.
		if ( preg_match( "/(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})Z/", $iTimestamp ) ) {
			$doc->ts = $iTimestamp;
		}
		else {
			if ( preg_match( "/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/", $iTimestamp ) ) {
				$doc->ts = preg_replace( "/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})/", "$1-$2-$3T$4:$5:$6Z", $iTimestamp );
			}
			else {
				global $wgDBtype;
				switch( $wgDBtype ) {
					case 'oracle' : 
						$doc->ts = preg_replace( "/(\d{2})-(\d{2})-(\d{4}) (\d{2}):(\d{2}):(\d{2}).(\d{6})/", "$3-$2-$1T$4:$5:$6Z", $iTimestamp ); 
						break;
					case 'postgres': 
						$doc->ts = preg_replace( "/(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})\+00/", "$1-$2-$3T$4:$5:$6Z", $iTimestamp ); 
						break;
				}
			}
		}

		$doc->setFieldBoost( 'title', 2 );
		$doc->setFieldBoost( 'text', 0.01 );
		return $doc;
	}

	/**
	 * Returns a unique id from parameter information.
	 * @param int $iID Id of an article.
	 * @param string $sPath Path to a file.
	 * @return string The unique ID in the index for a given document-id/-title/-path
	 */
	public function getUniqueId( $iID, $sPath = null ) {
		if ( ( $iID == -1 ) && empty( $sPath ) )
				throw new BsException( 'getUniqueId in BuildIndex has been called with $id == -1 and invalid $sPath: '.$sPath );
		/* md5 encoding of $path has several advantages
		 *    - almost unique and thus injective hash of any filepath
		 *    - 32 alphanumeric characters [0-9,a-f] => thus robust for encoding
		 *    - shorter and better performing than sha1 */
		return $this->sCustomerId.'-'.( ( $iID == -1 ) ? md5( $sPath ) : $iID );
	}

	/**
	 * Handles the search index log file.
	 * @param string $sMode delete, create, exists, is_writebale, write
	 * @param string $sData Text to write to file.
	 * @return bool False if there is no logfile, true if no mode is given, no return value otherwise.
	 */
	protected function logFile( $sMode = '', $sData = false ) {
		if ( empty( $this->sFilePathLogFile ) ) return false;
		if ( $sMode == '' ) return true;

		switch ( $sMode ) {
			case 'delete' :
				if ( $this->logFile( 'exists' ) )
					unlink( $this->sFilePathLogFile );
				return;
			case 'deleteLock' :
				if ( file_exists( $this->sFilePathLockFile ) )
					unlink( $this->sFilePathLockFile );
				return;
			case 'create' :
				touch( $this->sFilePathLogFile );
				return true;
			case 'createLock' :
				touch( $this->sFilePathLockFile );
				return true;
			case 'exists' :
				return file_exists( $this->sFilePathLogFile );
			case 'is_writable' :
				return is_writable( $this->sFilePathLogFile );
			case 'write' :
				if ( !$this->logFile( 'exists' ) ) {
					$this->logFile( 'create' );
				}
				if ( $this->logFile( 'is_writable' ) ) {
					$fp = fopen( $this->sFilePathLogFile, 'a' );
					fwrite( $fp, $sData );
					fclose( $fp );
				}
		}
	}

	/**
	 * Write log output
	 * @param string $sMode Information about the indexing area
	 * @param string $sMessage Message to log.
	 * @param string $sProgress Progress value between 0 and 100
	 * @return void Returns no value if mode was unlink and no value otherwise.
	 */
	public function writeLog( $sMode, $sMessage = '', $sProgress = '' ) {
		if ( $sMode == 'unlink' ) {
			if ( !$this->bCommandLineMode ) sleep( 2 );
			$this->writeLog( '__FINISHED__' );
			if ( !$this->bCommandLineMode ) sleep( 2 );
			unlink( $this->sFilePathIndexProgTxt );
			return;
		}

		$sMessage = addslashes( $sMessage );

		// todo: add as setting-param
		$bExtendedSearchIndexVerbose = true;
		if ( $bExtendedSearchIndexVerbose && $sMode != '__FINISHED__' ) {
			$output = "{$sMode}: {$sMessage} ...{$sProgress}%";
			if ( $this->logFile() ) {
				$this->logFile( 'write', "{$output}\n" ); // output to logFile
			}
			// output to console)
			if ( $this->bCommandLineMode ) {
				echo "{$output}\n";
			}
		}

		$line = '["'.$sMode.'", "'.$sMessage.'", "'.$sProgress.'"]';
		$fp = fopen( $this->sFilePathIndexProgTxt, 'w' ); // output one line to file, recreating file each time (for ajax progress bar)
		fwrite( $fp, $line );
		fclose( $fp );
	}

}