<?php
/**
 * Controls index building mechanism for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: BuildIndexMainControlMw.class.php 7685 2012-12-11 19:05:29Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Controls index building mechanism for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class BuildIndexMainControlMw extends BsDecorator {

	/**
	 * Reference to instance of base class.
	 * @var ExtendedSearchBaseMW Search base class.
	 */
	protected $oExtendedSearchBaseMw;
	/**
	 * Types to be indexed.
	 * @var array List of type keys (strings)
	 */
	protected $aTypes;
	/**
	 * Limit offset and number of documents to be indexed.
	 * @var string {Offset}{Limit}
	 */
	protected $sLimit;
	/**
	 * List of file types that can be indexed.
	 * @var array List of file extensions. Caution: these need to be known to the indexer.
	 */
	protected $aFiletypes;
	/**
	 * Maximum execution time per document to be indexed.
	 * @var int Execution time in seconds. 
	 */
	protected $iTimeLimit;

	/**
	 * Constructor for BuildIndexMainControlMw class
	 * @param ExtendedSearchBaseMw $oExtendedSearchBaseMw Reference to instance of base class.
	 * @param BsBuildIndexMainControl $oBsBuildIndexMainControl Instance to decorate.
	 * @param string $limit Limit offset and number of documents to be indexed. {Offset}{Limit}
	 * @param array $filetypes List of file types that can be indexed.
	 * @param array $types Types to be indexed.
	 * @param int $timeLimit Maximum execution time per document to be indexed.
	 */
	public function __construct( $oExtendedSearchBaseMw, $oBsBuildIndexMainControl, $sLimit = null, $aFiletypes = array(), $aTypes = null, $iTimeLimit = null ) {
		$this->oExtendedSearchBaseMw = $oExtendedSearchBaseMw;
		$this->sLimit = $sLimit;
		$this->aFiletypes = $aFiletypes;
		$this->processTypes( $aTypes );
		$this->iTimeLimit = $iTimeLimit;
		parent::__construct( $oBsBuildIndexMainControl ); // BsDecorator gets $instanceToDecorate
	}

	/**
	 * Getter for oExtendedSearchBaseMw
	 * @return ExtendedSearchBaseMw Reference to instance of base class.
	 */
	public function getExtendedSearchBaseMw() {
		return $this->oExtendedSearchBaseMw;
	}

	/**
	 * Prepare available types against default values.
	 * @param array $aTypes List of types.
	 * @return BuildIndexMainControlMw Return self for method chaining.
	 */
	protected function processTypes( $aTypes ) {
		if ( $this->aTypes !== null ) return $this;
		if ( !is_array( $aTypes ) ) $aTypes = array();
		if ( !isset( $aTypes['wiki'] ) ) {
			$aTypes['wiki'] = (bool) BsConfig::get( 'MW::ExtendedSearch::IndexTypesWiki' );
		}
		if ( !isset( $aTypes['repo'] ) ) {
			$aTypes['repo'] = (bool) BsConfig::get( 'MW::ExtendedSearch::IndexTypesRepo' );
		}
		if ( !isset( $aTypes['linked'] ) ) {
			$aTypes['linked'] = (bool) BsConfig::get( 'MW::ExtendedSearch::IndexTyLinked' );
		}
		if ( !isset( $aTypes['special-linked'] ) ) {
			$aTypes['special-linked'] = (bool) BsConfig::get( 'MW::ExtendedSearch::IndexTypesSpecialLinked' );
		}
		$this->aTypes = $aTypes;
		return $this;
	}

	/**
	 * Triggers index building for wiki
	 * @param string $mode I18N string that is used in progress bar
	 */
	public function buildIndexWiki( $mode ) {
		if ( $this->aTypes['wiki'] !== true ) return;
		$oBuildIndexInstance = new BuildIndexMwArticles( $this );
		$oBuildIndexInstance
			->setMode( $mode )
			->setTimeLimit( $this->iTimeLimit )
			->setLimit( $this->sLimit );
		$noOfResults = $oBuildIndexInstance->crawl();
		if ( !empty( $noOfResults ) ) {
			$oBuildIndexInstance->indexCrawledDocuments();
		}
	}

	/**
	 * Triggers index building for file repository
	 * @param string $mode I18N string that is used in progress bar
	 */
	public function buildIndexRepo( $mode ) {
		if ( $this->aTypes['repo'] !== true ) return;
		$oBuildIndexInstance = new BuildIndexMwRepository( $this );
		$oBuildIndexInstance
			->setMode( $mode )
			->setTimeLimit( $this->iTimeLimit )
			->setFileTypes( $this->aFiletypes )
			->setMaxDocSize( $this->iMaxDocSize )
			->setLimit( $this->sLimit );
		$noOfResults = $oBuildIndexInstance->crawl();
		if ( !empty( $noOfResults ) ) {
			$oBuildIndexInstance->indexCrawledDocuments();
		}
	}

	/**
	 * Triggers index building for external file repository
	 * @param string $sMode I18N string that is used in progress bar
	 */
	public function buildIndexExternalRepo( $sMode ) {
		if ( $this->aTypes['repo'] !== true ) return;
		$oBuildIndexInstance = new BuildIndexMwExternalRepository( $this );
		$oBuildIndexInstance
			->setMode( $sMode )
			->setTimeLimit( $this->iTimeLimit )
			->setFileTypes( $this->aFiletypes )
			->setMaxDocSize( $this->iMaxDocSize )
			->setLimit( $this->sLimit );
		$noOfResults = $oBuildIndexInstance->crawl();
		if ( !empty( $noOfResults ) ) {
			$oBuildIndexInstance->indexCrawledDocuments();
		}
	}

	/**
	 * Triggers index building for linked files
	 * @param string $sMode I18N string that is used in progress bar
	 * @param integer $iArticleID id of article to be indexed
	 */
	public function buildIndexLinked( $sMode, $iArticleID = null ) {
		if ( $this->aTypes['linked'] !== true ) return;
		$aErrorMessageKeys = array();
		$everythingsOk = BuildIndexMwLinked::areYouAbleToRunWithSystemSettings( $aErrorMessageKeys );
		if ( !$everythingsOk ) {
			foreach ( $aErrorMessageKeys as $key => $value ) {
				$this->writeLog( $sMode, wfMessage( $key )->plain(), 0 );
			}
			return;
		}
		$oBuildIndexInstance = new BuildIndexMwLinked( $this );
		$oBuildIndexInstance
			->setMode( $sMode )
			->setTimeLimit( $this->iTimeLimit )
			->setFileTypes( $this->aFiletypes )
			->setMaxDocSize( $this->iMaxDocSize )
			->setLimit( $this->sLimit )
			->setDbPrefix( BsCore::getInstance( 'MW' )->getAdapter()->DBprefix );
		$noOfResults = $oBuildIndexInstance->crawl( $iArticleID );
		if ( !empty( $noOfResults ) ) {
			$oBuildIndexInstance->indexCrawledDocuments();
		}
	}

	/**
	 * Triggers index building for linked files in special places like sharepoint
	 * @param string $sMode I18N string that is used in progress bar
	 */
	public function buildIndexSpecialLinked( $sMode ) {
		if ( $this->aTypes['special-linked'] !== true ) return;
		$aErrorMessageKeys = array();
		$everythingsOk = BuildIndexMwSpecialLinked::areYouAbleToRunWithSystemSettings( $aErrorMessageKeys );
		if ( !$everythingsOk ) {
			foreach ( $aErrorMessageKeys as $key => $value ) {
				$this->writeLog( $sMode, wfMessage( $key )->plain(), 0 );
			}
			return;
		}
		$oBuildIndexInstance = new BuildIndexMwSpecialLinked( $this );
		$oBuildIndexInstance
			->setMode( $sMode )
			->setTimeLimit( $this->iTimeLimit )
			->setFileTypes( $this->aFiletypes )
			->setMaxDocSize( $this->iMaxDocSize )
			->setLimit( $this->sLimit )
			->setDbPrefix( BsCore::getInstance( 'MW' )->getAdapter()->DBprefix );
		$noOfResults = $oBuildIndexInstance->crawl();
		if ( !empty( $noOfResults ) ) {
			$oBuildIndexInstance->indexCrawledDocuments();
		}
	}

}