<?php
/**
 * Controls repository index building mechanism for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: BuildIndexMwRepository.class.php 7070 2012-10-30 13:57:36Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * - initial commit
 */
/**
 * Controls repository index building mechanism for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class BuildIndexMwRepository extends BsAbstractBuildIndexFile {

	/**
	 * I18n key for general error message
	 */
	const S_ERROR_MSG_KEY = 'error-indexing-repo';
	/**
	 * Pointer to current database connection
	 * @var object Referenec to Database object
	 */
	protected $oDbr;
	/**
	 * List of documents to be indexed.
	 * @var resource Result of db query.
	 */
	protected $documentsDb = null;

	/**
	 * Constructor for BuildIndexMwLinked class
	 * @param BsBuildIndexMainControl $oBsBuildIndexMainControl Instance to decorate.
	 */
	public function __construct( $oMainControl ) {
		$this->oDbr = wfGetDB( DB_SLAVE );
		parent::__construct( $oMainControl );
	}

	/**
	 * Prepares list of documents to be indexed for wikis.
	 * @return int Number of documents to be indexed.
	 */
	public function crawl() {
	 $this->writeLog();
		$tables = 'image';
		$fields = array(
			'img_name',
			'img_minor_mime',
			'img_timestamp'
		);
		$clauses = 'img_major_mime = \'application\'';
		$options = $this->getLimitForDb();
		$this->documentsDb = $this->oDbr->select( $tables, $fields, $clauses, __METHOD__, $options );
		$this->totalNoDocumentsCrawled = $this->oDbr->numRows( $this->documentsDb );
		return $this->totalNoDocumentsCrawled;
	}

	/**
	 * Creates a document object that can be indexed by Solr.
	 * @param string $type Of type 'wiki', 'doc', 'ppt', 'xls', 'pdf', 'txt', 'html', 'sql', 'sh', ...
	 * @param string $img_name Filename of document to be indexed.
	 * @param string $fileText The body of the wiki-page or the document
	 * @param string $realPath Path to document if external (not in wiki). Might be empty or null
	 * @param unknown $ts Timestamp
	 * @return Apache_Solr_Document
	 */
	public function makeRepoDocument( $type, $img_name, &$text, $realPath, $ts ) {
		$doc = $this->oMainControl->makeDocument( 'repo', $type, $img_name, $text, -1, 999, $realPath, $ts );
		return $doc;
	}

	// duplicate of AbstractBuildIndexMwLinked
	// they have no predecessor in common in bluespice-mw
	// todo: externalize static, may be to AdapterMW
	/**
	 * Compares two timestamps
	 * @param string $timestamp1 MW timestamp
	 * @param string $timestamp2 MW timestamp
	 * @return bool True if timestamp1 is younger than timestamp2
	 */
	public function isTimestamp1YoungerThanTimestamp2( $timestamp1, $timestamp2 ) {
		$ts_unix1 = wfTimestamp( TS_UNIX, $timestamp1 );
		$ts_unix2 = wfTimestamp( TS_UNIX, $timestamp2 );
		return ( $ts_unix1 > $ts_unix2 );
	}

	/**
	 * Indexes all documents that were found in crawl() method.
	 */
	public function indexCrawledDocuments() {
		global $wgVersion;
		while ( $document = $this->oDbr->fetchObject( $this->documentsDb ) ) {
			$this->count++;
			$docType = $this->mimeDecoding( $document->img_minor_mime );
			if ( !isset( $this->aFileTypes[$docType] ) || ( $this->aFileTypes[$docType] !== true ) ) {
				$this->writeLog( ( 'Filetype not allowed: '.$docType.' ('.$document->img_name.')' ) );
				continue;
			}

			set_time_limit( $this->iTimeLimit );
			if ( $wgVersion < '1.18.0' ) {
				$oImage = Image::newFromName( $document->img_name );
				$path = $oImage->getFullPath();
			}
			else {
				$oTitle = Title::newFromText( $document->img_name, NS_FILE );
				$oFile = wfLocalFile( $oTitle );
				$oFileRepoLocalRef = $oFile->getRepo()->getLocalReference( $oFile->getPath() );
				if ( !is_null( $oFileRepoLocalRef ) ) {
					$path = $oFileRepoLocalRef->getPath();
				}
			}

			$repoFile = new SplFileInfo( $path ); // todo: check - might be urlencoded
			try {
				$repoFileSize = $repoFile->getSize(); // throws Exception if file does not exist
			}
			catch ( Exception $e ) {
				$this->writeLog( ( 'File does not exist: '.$document->img_name ) );
				continue;
			}
			if ( $this->sizeExceedsMaxDocSize( $repoFileSize ) ){
				$this->writeLog( ('File exceeds max doc size and will not be indexed: '.$document->img_name) );
				continue;
			}

			$repoFileRealPath = $repoFile->getRealPath();

			try {
				$uniqueIdForDocument = $this->oMainControl->getUniqueId( -1, $repoFileRealPath );
				$hitsDocumentInIndexWithSameUID = $this->oMainControl->oSearchService->search( 'uid:'.$uniqueIdForDocument, 0, 1 );
			}
			catch ( Exception $e ) {
				$this->writeLog( 'Error indexing file '.$document->img_name.' with errormessage '.$e->getMessage() );
				continue;
			}

			$timestampImage = $document->img_timestamp;

			// If already indexed and timestamp is not newer => don't index it!
			if ( $hitsDocumentInIndexWithSameUID->response->numFound != 0 ) {
				// timestamps have different format => compare function to equalize both
				$timestampIndexDoc = $hitsDocumentInIndexWithSameUID->response->docs[0]->ts;
				if ( !$this->isTimestamp1YoungerThanTimestamp2( $timestampImage, $timestampIndexDoc ) ) {
					$this->writeLog( ('Already in index: '.$document->img_name ) );
					continue;
				}
			}

			$text = '';
			try {
				$text = & $this->oMainControl->oSearchService->getFileText( $repoFileRealPath, $this->iTimeLimit );
			}
			catch ( Exception $e ) { // Exception can be of type Exception OR BsException
				$this->writeLog( ( 'Unable to extract document '.$document->img_name.', errormessage: '.$e->getMessage() ) );
				error_log( $e->getMessage() );
			}

			$doc = $this->makeRepoDocument( $docType, $document->img_name, $text, $repoFileRealPath, $timestampImage );
			$this->writeLog( $document->img_name );
			if ( $doc ) {
				// mode and ERROR_MSG_KEY are only passed for the case when addDocument fails
				$this->oMainControl->addDocument( $doc, $this->mode, self::S_ERROR_MSG_KEY );
			}
		}
	}

	/**
	 * Descructor for BuildIndexMwArticles class
	 */
	public function __destruct() {
		if ( $this->documentsDb !== null )
			$this->oDbr->freeResult( $this->documentsDb );
	}

}