<?php
/**
 * Indexer for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: SearchIndex.class.php 9548 2013-05-31 11:12:31Z smuggli $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * Indexer for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class SearchIndexMW implements ISearchIndex {

	/**
	 * Reference to current search service, e.g. Solr
	 * @var BsSearchService Current search service
	 */
	protected $oSearchService;
	/**
	 * Current search options as set by defaults and url params
	 * @var SearchOptions SearchOptions object
	 */
	protected $oSearchOptions;
	/**
	 * Search options as set by url params
	 * @var SearchRequestMW SearchRequestMW object
	 */
	protected $oSearchRequestMW;
	/**
	 * View that renders search results
	 * @var ViewSearchResultMW ViewSearchResultMW object
	 */
	protected $vSearchResultMW;
	/**
	 * Adapter for MediaWiki
	 * @var AdapterMW Current instance of adapter for MediaWiki
	 */
	protected $oAdapterMW;
	/**
	 * Mirror of wgScriptPath.
	 * @var string wgScriptPath
	 */
	protected $sScriptPath;
	/**
	 * Maximum number of facet items
	 * @var int Number
	 */
	protected $iMaxFacetLength = 20;
	/**
	 * SearchUriBuilder object
	 * @var SearchUriBuilder SearchUriBuilder object
	 */
	protected $oUriBuilder = null;

	/**
	 * Constructor for SearchIndexMW class
	 * @param BsSearchService $searchServiceObject Current search service
	 */
	public function __construct( $searchServiceObject ) {
		$this->oSearchService = $searchServiceObject;
		$this->oAdapterMW = BsCore::getInstance( 'MW' )->getAdapter();
		$this->sScriptPath = BsConfig::get( 'MW::ScriptPath' );
	}

	/**
	 * Shorten facet string.
	 * @param string $facet Name of facet
	 * @return string Shortened name of facet
	 */
	public function reduceMaxFacetLength( $facet ) {
		$facet = str_replace( '_', ' ', $facet );
		return BsStringHelper::shorten( 
					$facet,
					array(
						'max-length' => $this->iMaxFacetLength,
						'position' => 'middle'
					)
				);
	}

	/**
	 * This functions searches the index for a given search term
	 * @param BsSearchRequest $oSearchRequestCore Current search request
	 * @param array &$aMonitor Set of options.
	 * @return ViewSearchResultMW View that describes search results
	 */
	public function search( BsSearchRequest $oSearchRequestCore, &$aMonitor ) {
		$this->oSearchRequestMW = new SearchRequestMW( $oSearchRequestCore );
		$this->oSearchOptions = new SearchOptions( $this->oSearchRequestMW );
		$this->oUriBuilder = new SearchUriBuilderMW( $this->oSearchOptions );

		/* Jump to page */
		if ( BsConfig::get( 'MW::ExtendedSearch::JumpToTitle' )
			&& ( $this->oSearchOptions->getOption( 'titleExists' ) === true )
			&& ( $this->oSearchOptions->getOption( 'scope' ) == 'title' ) ) {
				$this->oAdapterMW->Out->redirect( $this->oSearchOptions->getOption( 'existingTitleObject' )->getFullURL() );
				throw new BsException('redirect');
		}
		if ( !$this->oSearchRequestMW->isSearchable() ) {
			if ( $this->oSearchRequestMW->sOrigin != '' && $this->oSearchOptions->getOption( 'searchStringOrig' ) == '' ) {
				return $this->createErrorMessageView( 'no_search_term' );
			}
			else {
				$vbe = new ViewBaseElement();
				$vbe->setAutoElement( false );
				return $vbe;
			}
		}

		// todo: $options sollte hier nicht als Variable auftauchen, sondern
		// mit $this->oSearchOptions->getOption() ausgelesen werden.
		$options = $this->oSearchOptions->getOptionsArray();

		if ( $this->oSearchService === null ) {
			$url = SpecialPage::getTitleFor( 'Search' )->getFullURL();
			$sParams = 'search='.urlencode( $options['searchStringRaw'] );
			$sParams .= ( $options['scope'] == 'title' ) ? '&go=' : '&fulltext=Search';
			foreach ( $options['namespaces'] as $namespace )
					$sParams .= "&ns{$namespace}=1";
			$url .= ( ( strpos( $url, '?' ) === false ) ? '?' : '&').$sParams;
			header( 'HTTP/1.1 307 Temporary Redirect' );
			header( 'Location: '.$url );
			exit(); // todo MKG 20110603 sauber beenden
		}

		/*
		 * WE HAVE
		 * - the SearchRequestMW AND the information: it's searchable
		 * - the SearchService
		 * WE NEED
		 * - a Query to transform the request into a query to the SearchService
		 *   This is done by SearchOptions that retrieves any Information
		 *   by ITSELF using SearchRequestMW
		 */
		$query = $this->oSearchOptions->getSolrQuery();
		try {
			// signature of BsSearchService, member function searchs($query, $offset = 0, $limit = 10, $params = array())
			$hits = $this->oSearchService->search( $query['searchString'], $query['offset'], $query['searchLimit'], $query['searchOptions'] );
			//$test = $this->oSearchService->search( 'test' ); //put $query into function search
		}
		catch ( Exception $e ) {
			// todo: explain more details to users
			return $this->createErrorMessageView( 'invalid-query' );
		}

		$hit_count = $hits->response->numFound;
		$max_score = $hits->response->maxScore;

		$bEscalateToFuzzy = ( $hit_count == 0 ); // boolean!
		// escalate to fuzzy
		if ( $bEscalateToFuzzy ) {
			$aFuzzyQuery = $this->oSearchOptions->getSolrFuzzyQuery(); // todo vgl. SearchClasses, #722
			try {
				$hits = $this->oSearchService->search( $aFuzzyQuery['searchString'], $aFuzzyQuery['offset'], $aFuzzyQuery['searchLimit'], $aFuzzyQuery['searchOptions'] );
			}
			catch ( Exception $e ) {
				return $this->createErrorMessageView( 'invalid-query' );
			}

			$hit_count = $hits->response->numFound;
			$max_score = $hits->response->maxScore;
		}

		$this->logSearch(
			$this->oSearchOptions->getOption( 'searchStringForStatistics' ),
			$hit_count,
			$this->oSearchOptions->getOption( 'scope' ),
			$this->oSearchOptions->getOptionBool( 'files' )
		);

		$this->vSearchResultMW = new ViewSearchResultMW();

		$this->vSearchResultMW->setOption( 'siteUri', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::MLT ) );

		if ( BsConfig::get( 'MW::ExtendedSearch::ShowSpell' ) ) {
			if ( $hit_count == 0 || $bEscalateToFuzzy ) {
				$this->vSearchResultMW->addSpell( 
					array(
						'script_path' => $this->sScriptPath,
						'sim' => $this->oSearchService->getSpellcheck( $this->oSearchOptions->getOption( 'searchStringRaw' ), $this->oSearchOptions->getSearchOptionsSim() ),
						'url' => $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::INPUT | SearchUriBuilderMW::MLT | SearchUriBuilderMW::ORDER_ASC_OFFSET )
					)
				);
			}
		}

		if ( BsConfig::get( 'MW::ExtendedSearch::ShowCreateSugg' ) ) {
			if ( $hit_count == 0 || $bEscalateToFuzzy || !$options['titleExists'] ) {
				$this->vSearchResultMW->addSuggest( 
					array(
						'search' => $this->oSearchOptions->getOption( 'searchStringRaw' ),
						'script_path' => $this->sScriptPath
					)
				);
			}
		}

		if ( $this->oSearchOptions->getOptionBool( 'morelikethis' ) ) {
			$sOut = '<p>'.wfMessage( 'bs-extendedsearch-mlt-results', '<a href="'.$options['mlt_link'].'">'.htmlspecialchars( $options['searchStringOrig'] )->plain().'</a>', $hit_count ).'<p>';
			$this->vSearchResultMW->addOutput( $sOut ); // todo: remove
		}
		else {
			$aMonitor['NoOfResultsFound'] = $hit_count;
			$aMonitor['SearchTerm']       = BsSearchService::sanitzeSearchString( $options['searchStringRaw'] ); // was searchStringOrig
			$aMonitor['EscalatedToFuzzy'] = $bEscalateToFuzzy;
		}

		if ( $hit_count == 0 ) return $this->vSearchResultMW;

	//--------- Navigation

		$aPaging = array();

		$searchLimit = $options['searchLimit'];

		$loopsCalculated = ( $hit_count / $searchLimit ) + 1;
		$this->vSearchResultMW->setOption( 'activePage', 1 );
		$url_offset = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::OFFSET );
		for( $i = 1; $i < $loopsCalculated; $i++ ) {
			$offset_step = ( ( $i - 1 ) * $searchLimit );
			if ( $offset_step == $options['offset'] ) {
				$this->vSearchResultMW->setOption( 'activePage', $i );
			}
			$aPaging[$i] = "{$url_offset}&search_offset={$offset_step}";
		}

		$this->vSearchResultMW->setOption( 'pages', $aPaging );

		if ( !$options['morelikethis'] ) {
			$sortTypes = array(
				'titleSort' => 'sort-title',
				'score'     => 'sort-relevance',
				'type'      => 'sort-type',
				'ts'        => 'sort-ts'
			);

			$sortActive = isset( $sortTypes[$options['order']] ) ? $options['order'] : 'score';
			$sortDirection = ( $options['asc'] == 'asc' ) ? 'asc' : 'desc';
			$sortUrl = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::MLT|SearchUriBuilderMW::ORDER_ASC_OFFSET );

			$this->vSearchResultMW->setOption( 'numFound', $hit_count );
			$this->vSearchResultMW->setOption( 'sorting', compact( 'sortTypes', 'sortActive', 'sortDirection', 'sortUrl' ) );
			//$this->vSearchResultMW->setOption( 'showCatFilter', (bool)( BsConfig::get( 'MW::ExtendedSearch::ShowCatFilter' ) ) );
		}

		//---------- end navigation

		//---------- begin facets

		if ( BsConfig::get( 'MW::ExtendedSearch::ShowFacets' ) ) {
			// possible sortoders: count, name, checked
			// 1 = desc, -1 = asc
			if ( BsConfig::get( 'MW::SortAlph' ) ) {
				$this->sortorder = array( 'name' => -1 );
			} else {
				$this->sortorder = array( 'count' => 1 );
			}
			$this->vSearchResultMW->setOption( 'showfacets', true );

			// --------------- begin facet namespace
			if ( !is_null( $hits->facet_counts->facet_fields->namespace ) ) {
				$vFacetBoxNamespaces = $this->vSearchResultMW->generateViewFacetBox();
				$vFacetBoxNamespaces->setOption( 'i18n-key-facet-title', 'bs-extendedsearch-facet-namespace' );
				$vFacetBoxNamespaces->setOption( 'uri-facet-delete', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::NAMESPACES|SearchUriBuilderMW::FILES ) );

				$facetNamespaceAll = array(); // alters to: array( 0 => array( 'checked' => true ), 1 => array( 'count' => 15 ), 999 => array( 'checked' => true, 'count' => 2 ) )

				if ( !empty( $options['namespaces'] ) ) {
					foreach ( $options['namespaces'] as $key => $namespace ) {
						$facetNamespaceAll[$namespace]['checked'] = true;
					}
				}

				$aFacetsNamespaceInHits = $hits->facet_counts->facet_fields->namespace; // stdClass object { 0 => (int)6, 999 => (int)1, _empty_ => (int)0 }
				foreach ( $aFacetsNamespaceInHits as $namespace => $count ) {
					if ( $this->oAdapterMW->checkNamespacePermission( $namespace, 'read' ) === false ) {
						unset( $facetNamespaceAll[$namespace] );
						continue;
					}
					if ( $namespace == '_empty_' ) continue;
					$facetNamespaceAll[$namespace]['count'] = $count;
				}

				foreach ( $facetNamespaceAll as $namespace => $attributes ) {
					if ( $namespace == '999' )
							$namespaceTitle = wfMessage( 'bs-extendedsearch-facet-namespace-files' )->plain();
					elseif ( $namespace == '998' )
							$namespaceTitle = wfMessage( 'bs-extendedsearch-facet-namespace-extfiles' )->plain();
					elseif ( $namespace == '0' )
							$namespaceTitle = wfMessage( 'bs-extendedsearch-facet-namespace-main' )->plain();
					else {
						$namespaceTitle = BsAdapterMW::getNamespaceName( $namespace, false );
						if ( empty( $namespaceTitle ) ) {
							unset( $facetNamespaceAll[$namespace] );
							continue;
							$namespaceTitle = wfMessage( 'bs-extendedsearch-facet-namespace-unknown' )->plain();
						}
						else {
							$namespaceTitle = $this->reduceMaxFacetLength( $namespaceTitle );
						}
					}
					$facetNamespaceAll[$namespace]['name'] = $namespaceTitle;
				}

				uasort( $facetNamespaceAll, array( $this, 'compareEntries' ) );

				$allNamespacesAvailableParamsOnly = array();
				foreach ( $facetNamespaceAll as $namespace => $attributes ) {
					if ( !isset( $attributes['count'] ) ) continue;
					$dataSet = array();
					if ( isset( $attributes['checked'] ) ) $dataSet['checked'] = true;
					if ( $namespace == '999' ) {
						$uri = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::NAMESPACES | SearchUriBuilderMW::FILES );
						$uri .= ( isset( $attributes['checked'] ) ) ? '&search_files=0' : '&search_files=1';
					}
					else {
						$uri = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::NAMESPACES );
					}
					foreach ( $facetNamespaceAll as $namespaceUrl => $attributesUrl ) {
						$bOwnUrlAndNotAlreadyChecked = ( ($namespace == $namespaceUrl) && !isset( $attributesUrl['checked'] ) );
						$bOtherUrlAndAlreadyChecked = ( ($namespace != $namespaceUrl) && isset( $attributesUrl['checked'] ) );
						if ( $bOwnUrlAndNotAlreadyChecked || $bOtherUrlAndAlreadyChecked ) {
							$uri .= '&na[]='.$namespaceUrl;
						}
					}
					$dataSet['uri'] = $uri;
					$dataSet['diff'] = 'na[]='.$namespace;
					$dataSet['name'] = $attributes['name'];
					$dataSet['count'] = ( isset( $attributes['count'] ) ) ? (int)$attributes['count'] : 0;
					$vFacetBoxNamespaces->addData( $dataSet );
					$allNamespacesAvailableParamsOnly[] = 'na[]='.$namespace;
				}

				$allNamespacesAvailableParamsOnly = implode( '&', $allNamespacesAvailableParamsOnly );
				$vFacetBoxNamespaces->setOption(
					'uri-facet-all',
					$this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::NAMESPACES|SearchUriBuilderMW::FILES ).'&'.$allNamespacesAvailableParamsOnly );
				$vFacetBoxNamespaces->setOption( 'uri-facet-all-diff', $allNamespacesAvailableParamsOnly );
			}
			// --------------- end facet namespaces

			// --------------- begin facet categories
			if ( !is_null( $hits->facet_counts->facet_fields->cat ) ) {
				$vFacetBoxCategories = $this->vSearchResultMW->generateViewFacetBox();
				$vFacetBoxCategories->setOption( 'i18n-key-facet-title', 'bs-extendedsearch-facet-category' );
				$vFacetBoxCategories->setOption( 'uri-facet-delete', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::CATS ) );

				$facetCategoriesAll = array();

				if ( !empty( $options['cats'] ) ) {
					foreach ( $options['cats'] as $key => $categoryName ) {;
						$facetCategoriesAll[$categoryName]['checked'] = true;
					}
				}

				$aFacetsCategoriesInHits = $hits->facet_counts->facet_fields->cat; // stdClass object { catName => (int)1, anotherCatName => (int)6, _empty_ => (int)1 }
				foreach ( $aFacetsCategoriesInHits as $cat => $count ) {
					if ( $cat == '_empty_' ) continue;
					$facetCategoriesAll[$cat]['count'] = $count;
				}

				foreach ( $facetCategoriesAll as $cat => $attributes ) {
					if ( $cat == 'notcategorized' )
							$catTitle = wfMessage( 'bs-extendedsearch-facet-uncategorized' )->plain();
					else $catTitle = $this->reduceMaxFacetLength( $cat );
					$facetCategoriesAll[$cat]['name'] = $catTitle;
				}

				uasort( $facetCategoriesAll, array( $this, 'compareEntries' ) );

				$allCategoriesAvailableParamsOnly = array();
				foreach ( $facetCategoriesAll as $cat => $attributes ) {
					$dataSet = array();
					if ( isset( $attributes['checked'] ) ) $dataSet['checked'] = true;
					$uri = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::CATS );
					foreach ( $facetCategoriesAll as $catUrl => $attributesUrl ) {
						$bOwnUrlAndNotAlreadyChecked = ( ( $cat == $catUrl ) && !isset( $attributesUrl['checked'] ) );
						$bOtherUrlAndAlreadyChecked = ( ( $cat != $catUrl ) && isset( $attributesUrl['checked'] ) );
						if ( $bOwnUrlAndNotAlreadyChecked || $bOtherUrlAndAlreadyChecked )
							$uri .= '&ca[]='.$catUrl;
					}
					$dataSet['uri'] = $uri;
					$dataSet['diff'] = 'ca[]='.$cat;
					$dataSet['name'] = $attributes['name'];
					$dataSet['count'] = ( isset( $attributes['count'] ) ) ? (int)$attributes['count'] : 0;
					$vFacetBoxCategories->addData( $dataSet );
					$allCategoriesAvailableParamsOnly[] = 'ca[]='.$cat;
				}

				$allCategoriesAvailableParamsOnly = implode( '&', $allCategoriesAvailableParamsOnly );
				$vFacetBoxCategories->setOption( 'uri-facet-all', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::CATS ).'&'.$allCategoriesAvailableParamsOnly );
				$vFacetBoxCategories->setOption( 'uri-facet-all-diff', $allCategoriesAvailableParamsOnly );
			}
			// --------------- end facet categories

			// --------------- begin facet type
			if ( !is_null( $hits->facet_counts->facet_fields->type ) ) {
				$vFacetBoxTypes = $this->vSearchResultMW->generateViewFacetBox();
				$vFacetBoxTypes->setOption( 'i18n-key-facet-title', 'bs-extendedsearch-facet-type' );
				$vFacetBoxTypes->setOption( 'uri-facet-delete', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::TYPE ) );

				$facetTypeAll = array();

				if ( !empty( $options['type'] ) )
					foreach ( $options['type'] as $key => $type )
						$facetTypeAll[$type]['checked'] = true;

				global $wgUser;
				$aFacetsTypeInHits = $hits->facet_counts->facet_fields->type;
				foreach ( $aFacetsTypeInHits as $type => $count ) {
					if ( $type == '_empty_' ) continue;
					if ( $type != 'wiki' && !$wgUser->isAllowed( 'searchfiles' ) ) {
						continue;
					}
					$facetTypeAll[$type]['count'] = $count;
				}

				foreach ( $facetTypeAll as $type => $attributes ) {
					$facetTypeAll[$type]['name'] = $this->reduceMaxFacetLength( $type );
				}

				uasort( $facetTypeAll, array( $this, 'compareEntries' ) );

				$allTypesAvailableParamsOnly = array();
				foreach ( $facetTypeAll as $type => $attributes ) {
					$dataSet = array();
					if ( isset( $attributes['checked'] ) ) $dataSet['checked'] = true;
					$uri = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::TYPE );
					foreach ( $facetTypeAll as $typeUrl => $attributesUrl ) {
						$bOwnUrlAndNotAlreadyChecked = ( ( $type == $typeUrl ) && !isset( $attributesUrl['checked'] ) );
						$bOtherUrlAndAlreadyChecked = ( ( $type != $typeUrl ) && isset( $attributesUrl['checked'] ) );
						if ( $bOwnUrlAndNotAlreadyChecked || $bOtherUrlAndAlreadyChecked )
							$uri .= '&ty[]='.$typeUrl;
					}
					$dataSet['uri'] = $uri;
					$dataSet['diff'] = 'ty[]='.$type;
					$dataSet['name'] = $attributes['name'];
					$dataSet['count'] = ( isset( $attributes['count'] ) ) ? (int)$attributes['count'] : 0;
					$vFacetBoxTypes->addData($dataSet);
					$allTypesAvailableParamsOnly[] = 'ty[]='.$type;
				}

				$allTypesAvailableParamsOnly = implode( '&', $allTypesAvailableParamsOnly );
				$vFacetBoxTypes->setOption( 'uri-facet-all', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::TYPE ).'&'.$allTypesAvailableParamsOnly );
				$vFacetBoxTypes->setOption( 'uri-facet-all-diff', $allTypesAvailableParamsOnly );
			}
			// --------------- end facet type

			// --------------- begin facet editor
			if ( !is_null( $hits->facet_counts->facet_fields->editor ) ) {
				$vFacetBoxEditors = $this->vSearchResultMW->generateViewFacetBox();
				$vFacetBoxEditors->setOption( 'i18n-key-facet-title', 'bs-extendedsearch-facet-editors' );
				$vFacetBoxEditors->setOption( 'uri-facet-delete', $this->oUriBuilder->buildUri(SearchUriBuilderMW::ALL, SearchUriBuilderMW::EDITOR ) );

				$facetEditorAll = array();

				if ( !empty( $options['editor'] ) )
					foreach ( $options['editor'] as $key => $editor )
						$facetEditorAll[$editor]['checked'] = true;

				$aFacetsEditorInHits = $hits->facet_counts->facet_fields->editor;
				foreach ( $aFacetsEditorInHits as $editor => $count ) {
					// todo: previously for _empty_ entry wfMessage( 'facet-noeditors' )
					// was displayed with count and without link
					// treat analogue to categories: index with field editor: anonymous
					if ( $editor == '_empty_' ) continue;
					$facetEditorAll[$editor]['count'] = $count;
				}

				foreach ( $facetEditorAll as $editor => $attributes ) {
					$facetEditorAll[$editor]['name'] = $this->reduceMaxFacetLength( $editor );
				}

				uasort( $facetEditorAll, array( $this, 'compareEntries' ) );

				$allEditorsAvailableParamsOnly = array();
				foreach ( $facetEditorAll as $editor => $attributes ) {
					//$editor = str_replace( ' ', '_', $editor );
					if ( $editor == 'unknown' ) $attributes['name'] = wfMessage( 'bs-extendedsearch-unknown' )->plain();
					$dataSet = array();
					if ( isset( $attributes['checked'] ) ) $dataSet['checked'] = true;
					$uri = $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::EDITOR );
					foreach ( $facetEditorAll as $editorUrl => $attributesUrl ) {
						$bOwnUrlAndNotAlreadyChecked = ( ( $editor == $editorUrl ) && !isset( $attributesUrl['checked'] ) );
						$bOtherUrlAndAlreadyChecked = ( ( $editor != $editorUrl ) && isset( $attributesUrl['checked'] ) );
						if ( $bOwnUrlAndNotAlreadyChecked || $bOtherUrlAndAlreadyChecked )
							$uri .= '&ed[]='.$editorUrl;
					}
					$editor = $editor;
					$dataSet['uri']   = $uri;
					$dataSet['diff']  = 'ed[]='.$editor;
					$dataSet['name']  = $attributes['name'];
					$dataSet['count'] = ( isset( $attributes['count'] ) ) ? (int)$attributes['count'] : 0;
					$vFacetBoxEditors->addData( $dataSet );
					$allEditorsAvailableParamsOnly[] = 'ed[]='.$editor;
				}

				$allEditorsAvailableParamsOnly = implode( '&', $allEditorsAvailableParamsOnly );
				$vFacetBoxEditors->setOption( 'uri-facet-all', $this->oUriBuilder->buildUri( SearchUriBuilderMW::ALL, SearchUriBuilderMW::TYPE ).'&'.$allEditorsAvailableParamsOnly );
				$vFacetBoxEditors->setOption( 'uri-facet-all-diff', $allEditorsAvailableParamsOnly );
			}
			// --------------- end facet editor
		}

		//---------- end facets

		//---------- begin results

		$hitlist = $hits->response->docs;

		if ( $options['morelikethis'] ) {
			foreach ( $hits->moreLikeThis as $key => $value ) {
				foreach ( $value->docs as $doc ) {
					$APSD = new Apache_Solr_Document();
					$APSD->__set( 'uid', $doc->uid[0] );
					$APSD->__set( 'title', $doc->title[0] );
					$APSD->__set( 'path', $doc->path[0] );
					$APSD->setMultiValue( 'cat', $doc->cat[0] );
					$APSD->__set( 'overall_type', $doc->overall_type[0] );
					$APSD->__set( 'type', $doc->type[0] );
					$APSD->__set( 'hwid', $doc->hwid[0] );
					$APSD->__set( 'namespace', $doc->namespace[0] );
					$APSD->__set( 'score', $doc->score[0] );
					$hitlist[] = $APSD;
				}
				break;
			}
		}

		if ( $options['format'] == 'json' ) {
			$hits = array();
			foreach ( $hitlist as $hit ) {
				$hitjson = '';
				$hitjson .= '{';
				$ns = $this->oAdapterMW->Lang->namespaceNames[$hit->namespace];
				$hitjson .= '"title":'.json_encode( ( $ns ? $ns.':' : '' ).$hit->title );
				$hitjson .= ', ';
				$hitjson .= '"cat":'.json_encode( $hit->cat );
				$hitjson .= ', ';
				$hitjson .= '"namespace":'.json_encode( $hit->namespace );
				$hitjson .= '}';
				$hits[] = $hitjson;
			}
			echo '['.implode( $hits, ', ' ).']';
			exit;
		}


		$this->vSearchResultMW->setResultEntryOptions( array(
			'showpercent'       => ( $this->oSearchOptions->getOption( 'showpercent' ) === true ), // todo: test
			'highlightsnippets' => ( $this->oSearchOptions->getOption( 'scope' ) != 'title' ),
			'showtime'          => BsConfig::get( 'MW::ExtendedSearch::ShowTime' )
			)
		);

		$sScriptPath = BsConfig::get( 'MW::ScriptPath' );

		$aImageLinks = array(
			'doc'     => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/word.gif" alt="Doc ' . wfMessage( 'bs-extendedsearch-file' )->plain() . '" /> ',
			'ppt'     => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/ppt.gif" alt="ppt ' . wfMessage( 'bs-extendedsearch-file' )->plain() . '" /> ',
			'xls'     => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/xls.gif" alt="xls ' . wfMessage( 'bs-extendedsearch-file' )->plain() . '" /> ',
			'pdf'     => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/pdf.gif" alt="pdf ' . wfMessage( 'bs-extendedsearch-file' )->plain() . '" /> ',
			'txt'     => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/txt.gif" alt="txt ' . wfMessage( 'bs-extendedsearch-file' )->plain() . '" /> ',
			'default' => '<img src="' . $sScriptPath . '/bluespice-mw/ext/ExtendedSearch/images/page.gif" alt="page" /> '
		);

		global $wgVersion;
		$bHideIcon   = BsConfig::get( 'MW::ExtendedSearch::HideWikiIcon' );

		foreach ( $hitlist as $document ) {
			if ( $options['morelikethis'] && ( $document->hwid == $options['mlt_id'] ) ) continue;

			$score_percent = 100 * ( sprintf( '%.2f', $document->score / $max_score ) );

			//Show Page Title and link it
			$si_icon = $aImageLinks['default'];

			if ( $document->namespace == '999' ) {
				$iNamespace = NS_FILE;
			}
			else {
				$iNamespace = $document->namespace;
			}
			$oTitle = Title::makeTitle( $iNamespace, $document->title );

			// external files will never exist for mediawiki
			if ( $document->namespace != '998' ) {
				if ( !$oTitle->exists() ) continue;
			}
			$oSkin = $this->oAdapterMW->User->getSkin();
			$searchlink = false;

			wfRunHooks( 'BSExtendedSearchFormatLink', array( &$searchlink, $document, $oSkin, &$si_icon ) );

			if ( !$searchlink ) {
				if ( $document->type == 'wiki' ) {
					if ( $bHideIcon ) {
						$si_icon = '';
					}

					if ( !$oTitle->userCan( 'read' ) ) continue;

					$searchlink = BsLinkProvider::makeLinkerLink( $oTitle, $sHtml = null, $aCustomAttribs = array(), $aQuery = array(), $aOptions = array( 'known' ) );

				}
				else if ( $this->oAdapterMW->User->isAllowed( 'searchfiles' ) ) {
					$si_icon = ( isset( $aImageLinks[$document->type] ) )
						? $aImageLinks[$document->type]
						: $aImageLinks['default'];

					$oLinker = new Linker();

					if ( $document->overall_type == 'repo' ) {
						if ( $wgVersion < '1.18.0' ) {
							$searchlink = $oLinker->makeMediaLinkObj( $oTitle );
						}
						else {
							$searchlink = Linker::makeMediaLinkObj( $oTitle );
						}
					}
					else if ( $document->overall_type == 'special-linked' ) {
						$si_title = $document->title;
						$si_link  = $document->path;
						if ( $wgVersion < '1.18.0' ) {
							$sLink = $oLinker->makeExternalLink( $si_link, $si_title, '' );
						}
						else {
							$sLink = Linker::makeExternalLink( $si_link, $si_title, '' );
						}
						$searchlink = str_replace( '<a', '<a target="_blank"', $sLink );
					}
					else {
						$si_title = $document->title;
						$si_link  = $document->path;

						$searchlink = '<a target="_blank" href="file:///' . $si_link . '">' . $si_title . '</a>';
					}
				}
				else continue;
			}

			//if ( BsConfig::get( 'MW::ExtendedSearch::ShowCatFilter' ) ) {
				if ( !isset( $document->cat ) ) {
					$catstr = '&nbsp;';
				}
				elseif ( is_array( $document->cat ) ) {
					$catlinks = array( );
					foreach ( $document->cat as $c ) {
						if ( $c == 'notcategorized' ) continue;
						$catstr = '';
						if ( !$options['morelikethis'] ) {
							//$catstr = '<a href="';
							$oTitle = Title::makeTitle( NS_CATEGORY, $c );
							$catstr = BsLinkProvider::makeLinkerLink( $oTitle, $oTitle->getText(), array(), array(), array('noclasses') );
							/*$catstr .= $this->oUriBuilder->buildUri( 
											SearchUriBuilderMW::ALL, SearchUriBuilderMW::OFFSET
											| SearchUriBuilderMW::MLT | SearchUriBuilderMW::CATS
							);
							$catstr .= '&ca[]='.$c.'">';*/
						}
						//$catstr .= str_replace( "_", " ", $c );
						//if ( !$options['morelikethis'] ) $catstr .= '</a>';
						$catlinks[] = $catstr;
					}
					$catstr = implode( ', ', $catlinks );
				}
				else {
					if ( $document->cat == 'notcategorized' ) $catstr = '&nbsp;';
					else {
						$catstr = '';
						if ( !$options['morelikethis'] ) {
							/*$catstr = '<a href="';
							$catstr .= $this->oUriBuilder->buildUri(
											SearchUriBuilderMW::ALL, SearchUriBuilderMW::OFFSET
											| SearchUriBuilderMW::MLT | SearchUriBuilderMW::CATS
							);
							$catstr .= '&ca[]='.$document->cat.'">';
							*/
							$oTitle = Title::makeTitle( NS_CATEGORY, $document->cat );
							$catstr = BsLinkProvider::makeLinkerLink( $oTitle, $oTitle->getText(), array(), array(), array('noclasses') );
						}
						//$catstr .= str_replace( "_", " ", $document->cat );
						//if ( !$options['morelikethis'] ) $catstr .= '</a>';
					}
				}
			//}

			// If text is empty no Notice will be thrown
			$aHighlightsnippets = null;
			if ( $options['scope'] != 'title' ) {
				$oHighlightData = $hits->highlighting->{$document->uid};
				if ( isset( $oHighlightData->textWord ) ) { 
					$aHighlightsnippets = $oHighlightData->textWord; 
				}
				else if ( isset( $oHighlightData->textReverse ) ) {
					$aHighlightsnippets = $oHighlightData->textReverse;
				}
			}

			$aResultEntryDataSet = array(
				'searchicon'        => $si_icon,
				'searchlink'        => $searchlink,
				'scorepercent'      => $score_percent,
				'timestamp'         => $this->oAdapterMW->Lang->timeanddate( $document->ts, true ),
				'catstr'            => $catstr,
				'highlightsnippets' => $aHighlightsnippets
			);

			$this->vSearchResultMW->addResultEntry( $aResultEntryDataSet );

		} // foreach $hitlist as $document

		// ----------end results ---------------

		return $this->vSearchResultMW;
	}

	/**
	 * Renders error message
	 * @param string $message I18N key of error message
	 * @return ViewBaseElement Renders error message.
	 */
	protected function createErrorMessageView( $message ) {
		$res = new ViewBaseElement();
		$res->setTemplate( '<div id="bs-es-searchterm-error">' . wfMessage( 'bs-extendedsearch-error' )->plain() . ': {message}</div>' );
		$res->addData( array( 'message' => wfMessage( 'bs-extendedsearch-' . $message )->plain() ) );
		return $res;
	}

	/**
	 * Detects whether two keys of an array are the same.
	 * @param array $array1 First array
	 * @param array $array2 Second array
	 * @param string $key (optional) key to be compared.
	 * @return int Comparison: 0 if equal, else + or -1
	 */
	protected function compareEntries( $array1, $array2, $key=false ) {
		if ( $key ) {
			if ( !isset( $array1[$key] ) && !isset( $array2[$key] ) ) return 0;
			if ( isset( $array1[$key] ) && !isset( $array2[$key] ) ) return -1;
			if ( !isset( $array1[$key] ) && isset( $array2[$key] ) ) return 1;
			if ( is_int( $array1[$key] ) && is_int( $array2[$key] ) )
				return ( ( (int)$array2[$key] ) - ( (int)$array1[$key] ) );
			return strcasecmp( (string)$array2[$key], (string)$array1[$key] );
		}
		else {
			foreach ( $this->sortorder as $sortkey => $asc ){
				$res = $this->compareEntries( $array1, $array2, $sortkey );
				if ( $res !== 0 ) return $res*$asc;
			}
		}
		return 0;
	}

	/**
	 * Stub function, usage unclear.
	 */
	protected function makeFacetBox() {}

	/**
	 * Writes a given search request to database log.
	 * @global User $wgUser Currently authenticated user.
	 * @param string $term Search term
	 * @param int $hit_count Number of hits
	 * @param string $scope What was the scope of the search?
	 * @param string $files Were files searched as well?
	 * @return bool always false.
	 */
	// CR MRG (25.06.11 00:55):  Check return value
	public function logSearch( $term, $hit_count, $scope, $files ) {
		if ( !BsConfig::get( 'Core::Search::Logging' ) ) return;
		global $wgUser, $wgDBtype;

		$dbw = wfGetDB( DB_MASTER );

		if ( $wgDBtype == 'postgres' ) {
			$term = pg_escape_string( $term );
		}
		else if ( $wgDBtype == 'oracle' ) {
			$term = addslashes( $term );
		}
		else {
			$term = mysql_real_escape_string( $term );
		}

		$user = ( BsConfig::get( 'Core::Search::LogUsers' ) ) ? $wgUser->getId() : '';
		$effectiveScope = ( $files ) ? $scope.'-files' : $scope;
		$data = array(
			'stats_term'  => $term,
			'stats_ts'    => date( 'YmdHis' ),
			'stats_user'  => $user,
			'stats_hits'  => $hit_count,
			'stats_scope' => $effectiveScope
		);
		$dbw->insert( 'bs_searchstats', $data );
	}

}