<?php
/**
 * Manages search options for ExtendedSearch for MediaWiki
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: SearchOptions.class.php 9078 2013-04-04 11:31:16Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2010 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v0.1
 * FIRST CHANGES
 */
/**
 * User-Input is stored in a object of class SearchRequest
 * All relevant data during searchtime can be found here in object of class SearchOptions
 * The class itself can generate all input for different solr-queries to be passed to
 * Apache_Solr_Service
 * @author Mathias
 */
/**
 * Manages search options for ExtendedSearch for MediaWiki
 * @package BlueSpice_Extensions
 * @subpackage ExtendedSearch
 */
class SearchOptions {

	/**
	 * Stores the original search string as entered into the form.
	 * @var string The original search string
	 */
	public static $searchStringRaw;
	/**
	 * Search options as set by url params
	 * @var SearchRequestMW SearchRequestMW object
	 */
	protected $oSearchRequest;
	/**
	 * Unique ID in order to distinguish this wiki in the index from others.
	 * @var string A unique ID
	 */
	protected $sCustomerId = null;
	/**
	 * Internal datastore
	 * @var array List of options.
	 */
	protected $aOptions = array(); // internal datastore
	/**
	 * Data being passed as $params to instance of BsSearchService, member function searchs($query, $offset = 0, $limit = 10, $params = array())
	 * @var array List of options.
	 */
	protected $aSearchOptions = array(); // data being passed as $params to instance of BsSearchService, member function searchs($query, $offset = 0, $limit = 10, $params = array())
	/**
	 * List of Solr query options
	 * @var array List of options.
	 */
	protected $aSolrQuery = null;
	/**
	 * List of Solr fuzzy query options
	 * @var array List of options.
	 */
	protected $aSolrFuzzyQuery = null;
	/**
	 * Adapter for MediaWiki
	 * @var AdapterMW Current instance of adapter for MediaWiki
	 */
	protected $oAdapterMW;

	/**
	 * Fetch unique ID for this wiki instance.
	 * @return string Unique ID
	 */
	protected function getCustomerId() {
		if ( $this->sCustomerId !== null ) return $this->sCustomerId;
		$customerId = BsConfig::get( 'Core::Search::CustomerID' );
		if ( empty( $customerId ) || strpos( $customerId, '?' ) !== false || strpos( $customerId, '*' ) !== false ) 
				throw new BsException( 'CustomerId must not be empty (settings extended search)' );
	
		wfRunHooks( 'BSExtendedSearchGetCustomerId', array( &$customerId ) );
		$this->sCustomerId = $customerId;
		return $this->sCustomerId;
	}

	/**
	 * Set a value for a certain option
	 * @param string $sKey Key of option
	 * @param mixed $vValue Value of option
	 */
	public function setOption( $sKey, $vValue ) {
		$this->aOptions[$sKey] = $vValue;
	}

	/**
	 * Get vaule for a certain option
	 * @param string $sKey
	 * @return mixed The value.
	 */
	public function getOption( $sKey ) {
		return $this->aOptions[$sKey];
	}

	/**
	 * Get complete options array
	 * @return array List of options 
	 */
	public function getOptionsArray(){
		return $this->aOptions;
	}

	/**
	 * Get search request
	 * @return SearchRequest 
	 */
	public function getSearchRequest(){
		return $this->oSearchRequest;
	}

	/**
	 * Get bool value of an option
	 * @param string $sKey Key for option
	 * @return bool Bool value of the option
	 */
	public function getOptionBool( $sKey ) {
		if ( !isset( $this->aOptions[$sKey] ) ) return false;
		return (bool) $this->aOptions[$sKey];
	}

	/**
	 * Constructor  for SearchOptions class
	 * @param BsSearchService $searchService Reference to search service.
	 */
	public function __construct( $oSearchRequest ) {
		$this->oAdapterMW     = BsCore::getInstance( 'MW' )->getAdapter();
		$this->oSearchRequest = $oSearchRequest;
		$this->readInSearchRequest();
	}

	/**
	 * Creates an array that can be used as a search query to Solr.
	 * @return array List of url parameters.
	 */
	public function getSolrQuery() {
		if ( $this->aSolrQuery !== null ) return $this->aSolrQuery;

		$this->aSolrQuery = array(
			'searchString'  => $this->aOptions['searchStringFinal'],
			'offset'        => $this->aOptions['offset'],
			'searchLimit'   => $this->aOptions['searchLimit'],
			'searchOptions' => $this->aSearchOptions,
			'isMltQuery'    => $this->aOptions['morelikethis']
		);

		return $this->aSolrQuery;
	}

	/**
	 * Creates an array that can be used as a search fuzzy query to Solr.
	 * @return array List of url parameters.
	 */
	public function getSolrFuzzyQuery() {
		if ( $this->aSolrFuzzyQuery !== null ) return $this->aSolrFuzzyQuery;
		// todo: ist dieses Vorgehen richtig? => Markus
		// Prefix, wildcard and fuzzy queries aren't analyzed, and thus won't match synonyms

		$fuzzySearchString = $this->aOptions['searchStringOrig'];
		$lastSignOfStringIsTilde = ( strrpos( $fuzzySearchString , '~' ) === ( strlen( $this->aOptions['searchStringOrig'] )-1 ) );
		$fuzzySearchString .= ( ( $lastSignOfStringIsTilde ) ? '' : '~' );
		$fuzzySearchString .= BsConfig::get( 'MW::ExtendedSearch::DefFuzziness' );
		$this->aOptions['searchStringForStatistics'] = $fuzzySearchString;

		if ( substr_count( $fuzzySearchString, '(' ) || substr_count( $fuzzySearchString, ')' ) ) {
			$fuzzySearchString = str_replace( '(', '\(', $fuzzySearchString );
			$fuzzySearchString = str_replace( ')', '\)', $fuzzySearchString );
		}

		$aFuzzySearch = array();
		$aFuzzySearch['searchStringInTitle'] = 'titleWord:('.$fuzzySearchString.')';
		$aFuzzySearch['searchStringInFulltext'] = 'textWord:('.$fuzzySearchString.')';

		$this->aOptions['searchStringFuzzy'] = ( $this->aOptions['scope'] == 'title' )
			? $aFuzzySearch['searchStringInTitle']
			: implode( ' OR ', $aFuzzySearch );

		$this->aSolrFuzzyQuery = array(
			'searchString'  => $this->aOptions['searchStringFuzzy'],
			'offset'        => $this->aOptions['offset'],
			'searchLimit'   => $this->aOptions['searchLimit'],
			'searchOptions' => $this->aSearchOptions
		);
		return $this->aSolrFuzzyQuery;
	}

	/**
	 * Creates an array that can be used as a similar search query to Solr.
	 * @return array List of url parameters.
	 */
	public function getSearchOptionsSim() {
		$customerId = $this->getCustomerId();
		return array(
				'fl' => 'title,namespace,score',
				'fq' => array('+overall_type:wiki', "+wiki:$customerId"),
				'spellcheck' => 'true',
				'spellcheck.collate' => 'true'
				//'sort'=>'namespace asc,titleSort asc',
			);
	}

	/**
	 * Retrieves parameters from defaults and search options.
	 */
	protected function readInSearchRequest() {
		$this->aOptions['bExtendedForm']             = $this->oSearchRequest->bExtendedForm;
		$this->aOptions['searchStringRaw']           = $this->oSearchRequest->sInput;
		$this->aOptions['searchStringOrig']          = BsSearchService::preprocessSearchInput( $this->oSearchRequest->sInput ); // includes SearchClasses::escapeForSolr
		$this->aOptions['searchStringWildcarded']    = BsSearchService::wildcardSearchstringOfOnlyOneWord( $this->aOptions['searchStringOrig'] );
		$this->aOptions['searchStringForStatistics'] = $this->aOptions['searchStringWildcarded'];
		$this->aOptions['searchStringComponents']    = array();

		self::$searchStringRaw = $this->aOptions['searchStringRaw'];

		$this->aOptions['morelikethis'] = (bool) $this->oSearchRequest->sMoreLikeThis;
		if ( $this->aOptions['morelikethis'] ) {
			$oTitle = Title::newFromID( $this->oSearchRequest->sId ); // Todo!

			$this->aOptions['mlt_title'] = $oTitle->getFullText();
			$this->aOptions['mlt_link']  = $oTitle->getFullURL();
			$this->aOptions['mlt_id']    = $this->oSearchRequest->sId; // Todo!

			$this->aSearchOptions['mlt']       = 'true';
			$this->aSearchOptions['mlt.fl']    = 'title,text'; // todo: titleWord, textWord
			$this->aSearchOptions['mlt.boost'] = 'true';
			$this->aSearchOptions['mlt.qf']    = 'title^10.0 text^1.0';
			$this->aSearchOptions['mlt.mindf'] = '1';
			$this->aSearchOptions['mlt.mintf'] = '1';
			$this->aSearchOptions['mlt.maxqt'] = '10';
			$this->aSearchOptions['mlt.minwl'] = '5';
			$this->aSearchOptions['mlt.count'] = '10';
			unset( $oTitle );
			//http://localhost:8080/solr/mlt?q=hwid:2084&start=0&rows=10&fl=title,score&mlt.fl=title,text&mlt.boost=true&mlt.qf=title%5E10.0&mlt.mindf=1&mlt.mintf=1&mlt.interestingTerms=details&mlt.maxqt=10&mlt.minwl=5
			//mlt.maxqt=10&mlt.minwl=5
		}

		// decide what is scope: 'text' or 'title'
		if ( $this->aOptions['morelikethis'] ) {
			$this->aOptions['scope'] = 'title';
		}
		else {
			$scope = BsConfig::get( 'MW::ExtendedSearch::DefScopeUser' ) == 'title' ? 'title' : 'text';
			if ( $this->oSearchRequest->sGo ) $scope = 'title';
			if ( $this->oSearchRequest->sSubmit ) $scope = 'text';
			if ( $this->oSearchRequest->sScope == 'title' ) $scope = 'title';
			if ( $this->oSearchRequest->sScope == 'text' ) $scope = 'text';
			$this->aOptions['scope'] = $scope;
		}

		// titleWord, titleReverse, textWord, texxtReverse sind "analyzed" (u.a. gestemmt)
		// Wird ein Query mit Wilcards ausgeführt, wird darauf vom Solr kein Analyzer angewendet => der Treffer liegt dann also wahrscheinlicher im Feld *Reverse

		$this->aOptions['searchStringComponents']['searchStringInTitle']    = 'titleWord:('.$this->aOptions['searchStringOrig'].') OR titleWord:('.$this->aOptions['searchStringOrig'].'*) OR titleReverse:('.$this->aOptions['searchStringWildcarded'].')';
		$this->aOptions['searchStringComponents']['searchStringInFulltext'] = 'textWord:('.$this->aOptions['searchStringOrig'].') OR textReverse:('.$this->aOptions['searchStringWildcarded'].')';

		$this->aOptions['searchStringFinal'] = ( $this->aOptions['scope'] == 'title' )
			? $this->aOptions['searchStringComponents']['searchStringInTitle']
			: implode( ' OR ', $this->aOptions['searchStringComponents'] );

		$customerId = $this->getCustomerId();

		$fq = array();
		$fq[] = '+wiki:('.$customerId.')';

		// $this->aOptions['namespaces'] HAS TO BE an array with numeric indices of type string!
		$this->aOptions['namespaces'] = $this->oSearchRequest->aNamespaces;

		if ( empty( $this->aOptions['namespaces'] ) ) {
			global $wgUser;
			if ( in_array( $this->oSearchRequest->sRequestOrigin, array( 'search_form_body', 'uri_builder', 'ajax' ) )
				|| $wgUser->getOption( 'searcheverything' ) == 1 ) {
				$this->aOptions['namespaces'] = array();
			}
			else {
				$this->aOptions['namespaces'] = array( '0' );

				wfRunHooks( 'BSExtendedSearchEmptyNamespacesElse', array( &$this ) );

				foreach ( $this->oAdapterMW->CanonicalNamespaceNames as $key => $value ) {
					if ( $this->oAdapterMW->User->getBoolOption( 'searchNs'.$key, false )
						|| isset( $this->oAdapterMW->NamespacesToBeSearchedDefault[$key] ) ) {
						$this->aOptions['namespaces'][] = $key;
					}
				}
				if ( $this->oSearchRequest->bSearchFiles === true ) {
					$this->aOptions['namespaces'][] = '998';
					$this->aOptions['namespaces'][] = '999';
				}
			}
		}

		/*
		 * If checkbox "search_files" is unchecked in the search form, this value is NOT PASSED AT ALL!
		 * In that case it is not possible to set $this->sFiles = $this->sDefaultSearchFiles, because the User probably wanted files not to be searched AND $this->sDefaultSearchFiles could be set to 'files'$
		 * Thus a hidden field was added to the search form with name="search_origin" and value="search_form_body"
		 * If that field can be read with the POST-data, the DefaultSearchFiles has to be omitted
		 */
		if ( $this->oSearchRequest->bSearchFiles === true ) {
			$this->aOptions['files'] = true;
		}
		else {
			$this->aOptions['files'] = false;
		}

		if ( !empty( $this->aOptions['namespaces'] ) || $this->aOptions['files'] === true ) {
			// todo (according to Markus): files should not be coded as namespace
			// => i.e. modify solr's schema
			$sFqNamespaces = '{!tag=na}+namespace:(';
			if ( empty( $this->aOptions['namespaces'] ) && ( $this->oSearchRequest->sRequestOrigin != 'uri_builder' ) ) {
				// if NO namespace selected search ALL namespaces
				// namespace 0 not in keys of wgCanonicalNamespaceNames
				// todo: just wondering: if NO namespace selected just SKIP +namespace(..)! Or is there a problem with namespace 999?
				$sFqNamespaces .= utf8_encode( '0 ' );
				foreach ( $this->oAdapterMW->CanonicalNamespaceNames as $na => $value ) {
					// filter -2, -1, 0
					if ( $na > 0 ) {
						if ( $this->oAdapterMW->checkNamespacePermission( $na, 'read' ) === false ) continue;
						$sFqNamespaces .= utf8_encode( $na.' ' );
					}
				}
			}
			else {
				foreach ( $this->aOptions['namespaces'] as $na ) {
					if ( $this->oAdapterMW->checkNamespacePermission( $na, 'read' ) === false ) continue;
					$sFqNamespaces .= utf8_encode( $na.' ' );
					if ( $na == '999' ) $filesAlreadyAddedInLoopBefore = true;
				}
			}
			if ( !isset( $filesAlreadyAddedInLoopBefore ) && $this->aOptions['files'] === true ) {
				$sFqNamespaces .= utf8_encode( '999 ' );
			}
			$fq[] = trim( $sFqNamespaces ).')';
		}

		// $this->aOptions['cats'] = $this->oSearchRequest->sCat; // string, defaults to '' if 'search_cat' not set in REQUEST
		$this->aOptions['cats'] = $this->oSearchRequest->sCategories; // array of strings or empty array

		if ( !empty( $this->aOptions['cats'] ) ) {
			if ( isset( $this->oSearchRequest->sOperator ) ) {
				switch ( $this->oSearchRequest->sOperator ) {
					case 'AND': 
						$sOperator = ' AND ';
						break;
					case 'OR':
						$sOperator = ' OR ';
						break;
					default:
						$sOperator = ' OR ';
				}
			}
			else {
				$sOperator = ' OR ';
			}
			$sFqCategories = '{!tag=ca}+cat:(';
			$sFqCategories .= implode( $sOperator, $this->aOptions['cats'] );
			$sFqCategories .= ')';
			$fq[] = $sFqCategories;
		}

		$this->aOptions['type'] = $this->oSearchRequest->sType;

		if ( !empty( $this->aOptions['type'] ) ) {
			$sFqType = '{!tag=ty}+type:(';
			$sFqType .= implode( ' ', $this->aOptions['type'] );
			$sFqType .= ')';
			$fq[] = $sFqType;
		}

		$this->aOptions['editor'] = $this->oSearchRequest->sEditor;

		if ( !empty( $this->aOptions['editor'] ) ) {
			// there may be spaces in name of editor. solr analyses those to two
			// terms (editor's names) thus we wrap the name into quotation marks
			// todo: better: in schema.xml define field editor not to be tokenized
			//       at whitespace
			// but: +editor:("Robert V" "Mathias S") is already split correctly!
			$sFqEditor = '{!tag=ed}+editor:("';
			$sFqEditor .= implode( '" "', $this->aOptions['editor'] );
			$sFqEditor .= '")';
			$fq[] = $sFqEditor;
		}

		$userLimit    = BsConfig::get( 'MW::ExtendedSearch::LimitResultsUser' );
		$defaultLimit = BsConfig::get( 'MW::ExtendedSearch::LimitResultDef' );
		$searchLimit  = ( $userLimit == 0 ) ? $defaultLimit : $userLimit;

		$this->aOptions['offset']       = $this->oSearchRequest->iOffset;
		$this->aOptions['order']        = $this->oSearchRequest->sOrder;
		$this->aOptions['asc']          = $this->oSearchRequest->sAsc;
		$this->aOptions['searchLimit']  = ( $searchLimit == 0 ) ? 10 : $searchLimit;
		$this->aOptions['contextlines'] = $this->oAdapterMW->User->getOption( 'contextlines', 5 );
		$this->aOptions['titleExists']  = $this->titleExists( $this->oSearchRequest->sInput );
		$this->aOptions['format']       = $this->oSearchRequest->sFormat;
		$this->aOptions['showpercent']  = BsConfig::get( 'MW::ExtendedSearch::ShowPercent' );

		$this->aSearchOptions['fl']          = 'wiki,uid,hwid,type,title,overall_type,path,namespace,cat,score,ts';
		$this->aSearchOptions['fq']          = $fq;
		$this->aSearchOptions['sort']        = $this->aOptions['order'].' '.$this->aOptions['asc'];
		$this->aSearchOptions['hl']          = 'on';
		$this->aSearchOptions['hl.fl']       = 'textWord, textReverse';
		$this->aSearchOptions['hl.snippets'] = BsConfig::get( 'Core::Search::HighlightSnippets' );

		if ( BsConfig::get( 'MW::ExtendedSearch::ShowFacets' ) ) {
			$this->aSearchOptions['facet']          = 'on';
			$this->aSearchOptions['facet.sort']     = 'false';
			$this->aSearchOptions['facet.field']    = array( '{!ex=na}namespace', '{!ex=ca}cat', '{!ex=ov}overall_type', '{!ex=ty}type', '{!ex=ed}editor' );
			$this->aSearchOptions['facet.mincount'] = '1';
			$this->aSearchOptions['facet.missing']  = 'true';
		}
	}

	/**
	 * For a given SearchInput (by the user) the Existence of an article with exactly this title is evaluated.
	 * @param String $sParamSearchInput
	 * @return Title
	 */
	public function titleExists( $sParamSearchInput ) {
		if ( !isset( $sParamSearchInput ) ) return false;
		if ( $sParamSearchInput === false ) return false;
		if ( $sParamSearchInput === '' ) return false;
		if ( $sParamSearchInput === null ) return false;
		$paramSearchInputTrimmed = trim( $sParamSearchInput );
		if ( empty( $paramSearchInputTrimmed ) ) return false;

		/* Normalize $sParamSearchInput first:
		 * - get rid of leading or trailing whitespace
		 * - get rid of characters that are not permitted in the title (by mediawiki)
		 * - get rid of more than one space at a time
		 */
		$notAllowedInTitle = array( '#', '<', '>', '[', ']', '|', '{', '}' ); // see python.eventscripts.com/pages/MediaWiki_Naming_conventions_%28technical_restrictions%29
		$thisTitleMightExist = trim( str_replace( $notAllowedInTitle, ' ', $sParamSearchInput ) );
		do {
			$beforeStrReplace = $thisTitleMightExist;
			$thisTitleMightExist = str_replace( '  ', ' ', $thisTitleMightExist );
		}
		while ( $beforeStrReplace != $thisTitleMightExist );

		$oTitle = Title::newFromText( $thisTitleMightExist );
		if ( ( $oTitle !== null ) && $oTitle->exists() ) {
			$this->aOptions['existingTitleObject'] = &$oTitle;
			return true;
		}
		// If first attempt to create a Title-Object without success...
		// ... remove leading and trailing '"'-characters (solves Ticket#2010062310000113)
		$oTitle = Title::newFromText( trim( $thisTitleMightExist, '"' ) );
		if ( ( $oTitle !== null ) && $oTitle->exists() ){
			$this->aOptions['existingTitleObject'] = &$oTitle;
			return true;
		}
		return false;
	}

}