<?php
/**
 * Renders the ExtendedSearch results page.
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Mathias Scheer <scheer@hallowelt.biz>
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @author     Stephan Muggli <muggli@hallowelt.biz>
 * @version    $Id: view.SearchResultMW.php 9296 2013-05-02 12:02:08Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/**
 * <div style:"display: none"> mit Hilfen
 */
/**
 * This view renders the ExtendedSearch results page.
 * @package    BlueSpice_Extensions
 * @subpackage ExtendedSearch 
 */
class ViewSearchResultMW extends ViewBaseElement {

	/**
	 * Contains the page output.
	 * @var string HTML output.
	 */
	protected $out = '';
	/**
	 * List of messages to be rendered.
	 * @var array List of strings.
	 */
	protected $aMessages = array();
	/**
	 * Contain view for search entry.
	 * @var ViewExtendedSearchResultEntry View for search entry.
	 */
	protected $vResultEntryView = null;
	/**
	 * List of facet boxes.
	 * @var array List of ViewExtendedSearchFacetBox.
	 */
	protected $aFacetBoxes = array();

	/**
	 * Create a new result entry view and add it to self.
	 * @return ViewExtendedSearchResultEntry The view that was added.
	 */
	protected function resultEntryView() {
		if ( $this->vResultEntryView === null ){
			$this->vResultEntryView = new ViewExtendedSearchResultEntry();
			$this->addItem( 'resultentries', $this->vResultEntryView );
		}

		return $this->vResultEntryView;
	}

	/**
	 * Set the options of the current esult entry view.
	 * @param array $aOptions List of options to be set.
	 */
	public function setResultEntryOptions( array $aOptions ) {
		$this->resultEntryView()->setOptions( $aOptions );
	}

	/**
	 * Adds data to the current result entry view.
	 * @param array $aDataSet List of result items.
	 */
	public function addResultEntry( $aDataSet ) {
		$this->resultEntryView()->addData( $aDataSet );
	}

	/**
	 * Adds a ceate/suggest section.
	 * @param array $aOptions Parameters for this section.
	 */
	public function addSuggest( $aOptions ) {
		$vSuggest = new ViewSearchSuggest();
		$vSuggest->setOptions( $aOptions );
		$this->addItem( $vSuggest );
	}

	/**
	 * Adds a ceate/suggest section.
	 * @param array $aOptions Parameters for this section.
	 */
	public function addSpell( $aOptions ) {
		$vSpell = new ViewSpell();
		$vSpell->setOptions( $aOptions );
		$this->addItem( $vSpell );
	}

	/**
	 * Adds a message to be displayed.
	 * @param string $key Key for the message.
	 * @param string $message The message in HTML.
	 */
	public function addMessage( $key, $message ) {
		$this->aMessages[$key] = $message;
	}

	/**
	 * Adds additional output to page.
	 * @param string $outputToAdd HTML that shall be displayed.
	 */
	public function addOutput( $outputToAdd ) {
		$this->out .= $outputToAdd;
	}

	/**
	 * Prepares a message for direct output.
	 * @param string $message The message that should be prepared.
	 * @return string The modified message.
	 */
	protected function secureMessage( $message ) {
		$search = array( "&lt;b&gt;", "&lt;/b&gt;", "&lt;i&gt;", "&lt;/i&gt;" );
		$replace = array( "<b>", "</b>", "<i>", "</i>" ); // these tags may be contained in i18n messages
		$secure = htmlentities( $message, ENT_QUOTES, 'UTF-8' );

		return str_replace( $search, $replace, $secure ); // primitive whitelisting
	}

	/**
	 * This method actually generates the output
	 * @return string HTML output
	 */
	public function execute( $param = false ) {
		$out = array();
		$out[] = parent::execute( $param = false );
		foreach ( $this->aMessages as $message )
			$out[] = '<p>'.$this->secureMessage( $message )."</p>\n"; // primitive whitelisting

		$out[] = $this->out;
		if ( $this->vResultEntryView !== null ) {
			$out[] = '<div id="bs-extendedsearch-spinner"></div>';
			$out[] = '<hr /><br />';
			if ( $this->getOption( 'siteUri' ) ) {
				$out[] = BsXml::simpleTag( 'div',
					array(
						'id'=> 'bs-extendedsearch-siteuri',
						'siteuri' => $this->getOption( 'siteUri' )
					),
					'',
					false );
			}
			$out[] = '<div id="bs-extendedsearch-filters-results-paging">';
			if ( $this->getOption( 'showfacets' ) ) {
				$filterHeadlineInner = BsXml::simpleTag('div', array(
						'id' => 'bs-extendedsearch-filter-headline-inner' ),
					wfMessage( 'bs-extendedsearch-filter' )->plain() );
				$filterHeadlineOuter = BsXml::simpleTag('div', array(
						'class' => 'bs-extendedsearch-filter-headline-outer' ),
					$filterHeadlineInner );
				$filterBoxes = '';
				foreach ( $this->aFacetBoxes as $box ) {
					$filterBoxes .= $box->execute();
				}
				$filterBoxes = BsXml::simpleTag('div', array(
						'id' => 'bs-extendedsearch-all-filter-boxes' ),
					$filterBoxes );
				$out[] = BsXml::simpleTag('div', array(
						'id' => 'bs-extendedsearch-filters' ),
					$filterHeadlineOuter.$filterBoxes );
			}
			$cachePaging = $this->getPaging();
			$upperPaging = BsXml::simpleTag( 'div', array(
					'class' => 'bs-extendedsearch-default-line bs-extendedsearch-paging',
					'id' => 'bs-extendedsearch-paging-upper' ),
				$cachePaging );
			$lowerPaging = BsXml::simpleTag( 'div', array(
					'class'=>'bs-extendedsearch-paging',
					'id'=>'bs-extendedsearch-paging-lower' ),
				$cachePaging );
			$sortingBar = BsXml::simpleTag('div', array(
					'class' => 'bs-extendedsearch-sorting-bar bs-extendedsearch-default-textspacing' ),
				$this->getSortingBar() );
			$results = BsXml::simpleTag('div', array(
					'id'=>'bs-extendedsearch-results' ),
				$this->vResultEntryView->execute() );
			$out[] = ( $this->getOption( 'showfacets' ) )
					? '<div id="bs-extendedsearch-results-paging" class="bs-extendedsearch-results-with-facets">'
					: '<div id="bs-extendedsearch-results-paging">';
			$out[] = $upperPaging.$sortingBar.$results.$lowerPaging;
			$out[] = '  </div>'; // bs-extendedsearch-results-paging
			$out[] = '</div>'; // bs-extendedsearch-filters-results-paging
			$out[] = '<div id="bs-extendedsearch-results-finalizer"></div>';
		}
		// Placeholder for SearchUsers, SearchProfiles, SearchHelpdesk etc
		$out = implode( "\n", $out ).'<br /><br />'; // for space between results and following box (if Core::Search::BoxOnTop set to false)

		return $out;
	}

	/**
	 * Displays a single paging box, either with page number or with prev/next arrows.
	 * @param string $pageNo Label of the page. Number or prev/next arrows.
	 * @param string $url Link to be followed when clicked.
	 * @param bool $active Is this box currently selected?
	 * @param bool $arrows Is it an arrow box?
	 * @return string HTML of paging box.
	 */
	protected function makePagingDiv( $pageNo, $url = '', $active = false, $arrows = false ) {
		$aStyleClasses = array(
			'bs-extendedsearch-paging-no',
			'bs-extendedsearch-default-textspacing'
		);
		if ( $arrows ) $aStyleClasses[] = 'bs-extendedsearch-paging-arrows';
		if ( $active ) $aStyleClasses[] = 'bs-extendedsearch-paging-no-active';
		$out = $pageNo;
		if ( $active ) $out = "<b>{$out}</b>";
		$out = '<div class="'.implode( ' ', $aStyleClasses ).'">'.$out.'</div>';
		if ( !$active && $url !== '' ) $out = "<a href=\"{$url}\">{$out}</a>";

		return $out;
	}

	/**
	 * Renders the paging bar.
	 * @return string HTML of paging bar.
	 */
	protected function getPaging() {
		$out = array();
		$aPaging = $this->getOption( 'pages' );
		$pageActive = (int) $this->getOption( 'activePage' );
		$firstPageToDisplay = ( $pageActive > 5 ) ? $pageActive - 5 : 1;
		end( $aPaging );
		$lastPage = key( $aPaging );
		$lastPageToDisplay = ( $lastPage - $pageActive > 5 ) ? $pageActive + 5 : $lastPage;
		$out[] = ( $pageActive > 1 )
			? $this->makePagingDiv( htmlspecialchars( '<' ), $aPaging[$pageActive - 1], false, true )
			: $this->makePagingDiv( htmlspecialchars( '<' ), '', false, true );
		if ( $firstPageToDisplay > 1 )
				$out[] = $this->makePagingDiv( 1, $aPaging[1] );
		if ( $firstPageToDisplay > 2 )
				$out[] = '<div class="bs-extendedsearch-paging-dots bs-extendedsearch-default-textspacing">...</div>';
		foreach ( $aPaging as $page => $url ) {
			if ( $firstPageToDisplay > $page ) continue;
			if ( $lastPageToDisplay < $page ) break;
			$out[] = $this->makePagingDiv( $page, $url, ( $page == $pageActive ) ); // (($page == $lastKeyInAPaging) || $page == 1)
		}
		if ( $lastPageToDisplay + 1 < $lastPage )
				$out[] = '<div class="bs-extendedsearch-paging-dots bs-extendedsearch-default-textspacing">...</div>';
		if ( $lastPageToDisplay < $lastPage )
				$out[] = $this->makePagingDiv( $lastPage, $aPaging[$lastPage] );
		$out[] = ( $pageActive < $lastPage )
				? $this->makePagingDiv( htmlspecialchars( '>' ), $aPaging[$pageActive + 1], false, true )
				: $this->makePagingDiv( htmlspecialchars( '>' ), '', false, true );
		$out = implode( '<div class="bs-extendedsearch-paging-spacer"></div>', $out );

		return $out;
	}

	/**
	 * Renders the sorting bar.
	 * @return string HTML of sorting bar.
	 */
	protected function getSortingBar() {
		// compact('sortTypes', 'sortActive', 'sortDirection', 'sortUrl')
		extract( $this->getOption( 'sorting' ) );
		$out = '<span style="float: left;">';
		$out .= $this->getNumberofPageItems() . ' ';
		$out .= wfMessage( 'bs-extendedsearch-sort-by' )->plain();
		$out .= '&nbsp;';
		// 'titleSort', 'score', 'type', 'ts'
		$iItems = count( $sortTypes );
		$iNum = 1;
		foreach ( $sortTypes as $sort => $i18nKey ) {
			$active = ( $sort == $sortActive );
			if ( $active ) {
				$direction = ( $sortDirection == 'asc' ) ? 'desc' : 'asc';
				$icon = '<img src="';
				$icon .= BsConfig::get( 'MW::ScriptPath' );
				$icon .= '/bluespice-mw/ext/ExtendedSearch/images/';
				$icon .= ( $sortDirection == 'asc' ) ? 'arrow_up.png' : 'arrow_down.png';
				$icon .= '" border="0"/>&nbsp;';
			}
			else {
				// $direction = $sortDirection todo: think it over: if sort order is changed from score to time, the order should be reset!
				$direction = ( in_array( $sort, array( 'titleSort', 'type' ) ) ) ? 'asc' : 'desc';
				$icon = '';
			}
			if ( $active ) $out .= '<b>';
			$out .= '<a href="'.$sortUrl.'&search_asc='.$direction.'&search_order='.$sort.'">';
			$out .= wfMessage( 'bs-extendedsearch-' . $i18nKey )->plain() . '&nbsp;'.$icon;
			$out .= '</a>';
			if ( $active ) $out .= '</b>';
			if ( $iNum < $iItems ) {
				$out .= '|&nbsp;';
			}
			$iNum++;
		}

		$out .= '</span>';

		//if ( $this->getOption( 'showCatFilter' ) ) {
			$out .= '<span id="bs-extendedsearch-category-filter-title">' . wfMessage( 'bs-extendedsearch-category-filter' )->plain() . '</span>';
		//}

		return $out;
	}

	/**
	 * Generates a facet box view.
	 * @return ViewExtendedSearchFacetBox Generated facet box view.
	 */
	public function generateViewFacetBox() {
		$newBox = new ViewExtendedSearchFacetBox();
		$this->aFacetBoxes[] = & $newBox;

		return $newBox;
	}

	/**
	 * Returns the range of numbers which articles are displayed
	 * @return string range.
	 */
	public function getNumberofPageItems() {
		$iNumOfResults = BsConfig::get( 'MW::ExtendedSearch::LimitResultDef' );
		$iBegin        = ( ( $this->getOption( 'activePage' ) - 1 ) * $iNumOfResults ) + 1;
		$iEnd          = $this->getOption( 'activePage' ) * $iNumOfResults;

		if ( $this->getOption( 'numFound' ) < $iEnd ) {
			$iEnd = $this->getOption( 'numFound' );
		}

		return $iBegin .' - ' . $iEnd . ' ' . wfMessage( 'bs-extendedsearch-outof' )->plain() .
				' ' . $this->getOption( 'numFound' ) . ' ' . wfMessage( 'bs-extendedsearch-result-caption' )->plain() . ' ';
	}

}