<?php

/**
 * BlueSpice for MediaWiki
 * Extension: PageAccess
 * Description: Controls access on page level.
 * Authors: Marc Reymann
 *
 * Copyright (C) 2010 Hallo Welt! – Medienwerkstatt GmbH, All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * http://www.gnu.org/copyleft/gpl.html
 *
 * For further information visit http://www.blue-spice.org
 *
 * Version information
 * $LastChangedDate: 2013-06-12 12:13:02 +0200 (Mi, 12. Jun 2013) $
 * $LastChangedBy: pwirth $
 * $Rev: 9690 $
 * $Id: PageAccess.class.php 9690 2013-06-12 10:13:02Z pwirth $
 */
/* Changelog
 * v1.20.0
 * - initial release
 */

/**
 * PageAccess adds a tag, used in WikiMarkup as follows:
 * Grant exclusive access to group "sysop": <bs:pageaccess groups="sysop" />
 * Separate multiple groups by commas.
 */
class PageAccess extends BsExtensionMW {

	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['PageAccess'] = dirname( __FILE__ ) . '/PageAccess.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::PARSERHOOK; //SPECIALPAGE/OTHER/VARIABLE/PARSERHOOK
		$this->mInfo = array(
			EXTINFO::NAME => 'PageAccess',
			EXTINFO::DESCRIPTION => 'Controls access on page level.',
			EXTINFO::AUTHOR => 'Marc Reymann',
			EXTINFO::VERSION => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS => 'stable',
			EXTINFO::URL => 'http://www.hallowelt.biz',
			EXTINFO::DEPS => array( 'bluespice' => '1.21.0' )
		);
		$this->mExtensionKey = 'MW::PageAccess';

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	protected function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );
		$this->setHook( 'ArticleSave' );
		$this->setHook( 'ParserFirstCallInit' );
		$this->setHook( 'userCan' );
		$this->setHook( 'BSInsertMagicAjaxGetData', 'onBSInsertMagicAjaxGetData' );
		wfProfileOut( 'BS::'.__METHOD__ );
	}

	public function onArticleSave( &$article, &$user, &$text, &$summary, $minor, $watchthis, $sectionanchor, &$flags, &$status ) {
		global $wgVersion;
		# Prevent user from locking himself out of his own page
		$oEditInfo = $article->prepareTextForEdit( $text, null, $user ); 
		$sAccessGroups = $oEditInfo->output->getProperty( 'bs-page-access' );
		if ( !$this->isUserAllowed( $user, $sAccessGroups ) ) {
			if ( version_compare( $wgVersion, '1.18.0', '<' ) ) {
				throw new MWException( "Lockout prevented in " . __METHOD__ );
				#$status->fatal( 'bs-pageaccess-error-not-part-of-given-groups' );
				return false;
			}
			else {
				$err[0] = 'bs-pageaccess-error-not-part-of-given-groups';
				throw new PermissionsError( 'edit', array( $err ) ); # since MW 1.18
				return false;
			}
		}
		return true;
	}

	public function isUserAllowed( $oUser, $sAccessGroups) {
		if ( !$sAccessGroups ) return true;
		$aAccessGroups = array_map("trim", explode( ',', $sAccessGroups ) );
		wfRunHooks( 'BSPageAccessAddAdditionalAccessGroups', array( &$aAccessGroups ) );
		#error_log("\$aAccessGroups: ".var_export($aAccessGroups,1));
		$aUserGroups = $oUser->getEffectiveGroups();
		#error_log("\$aUserGroups: ".var_export($aUserGroups,1));
		return (bool) array_intersect( $aAccessGroups, $aUserGroups );
	}
	
	public function onParserFirstCallInit( &$parser ) {
		$parser->setHook( 'bs:pageaccess', array( &$this, 'onTagBsPageAccess' ) );
		return true;
	}

	public function onUserCan( $title, $user, $action, &$result ) {
		#if ( $title->isSpecialPage() ) return true;
		if( !in_array($action, array('read', 'edit', 'delete', 'move')) ) return true;
		
		$dbr = wfGetDB( DB_SLAVE );
		$sAccessGroups = $dbr->selectField( 'page_props', 'pp_value',
				array( 'pp_page' => $title->getArticleID(), 'pp_propname' => 'bs-page-access' ), __METHOD__ );
		if ( $this->isUserAllowed( $user, $sAccessGroups ) ) return true;
		#$result = array( 'badaccess-groups' );
		$result = false;
		return false;
	}
	
	/**
	 * 
	 * @param type $input
	 * @param string $args
	 * @param Parser $parser
	 * @return string
	 */
	public function onTagBsPageAccess( $input, $args, $parser ) {
		//ignore access tag on mainpage or it will break all ajax calls without title param
		if( $parser->getTitle()->equals( Title::newMainPage() ) === true ) return '';

		$parser->disableCache();

		if ( !isset( $args['groups'] ) ) {
			$oErrorView = new ViewTagError( wfMessage( 'bs-pageaccess-error-no-groups-given' )->escaped() );
			return $oErrorView->execute();
		}

		$sOldAccessGroups = $parser->getOutput()->getProperty( 'bs-page-access' );
		if ( $sOldAccessGroups ) $args['groups'] = $sOldAccessGroups . "," . $args['groups'];
		$parser->getOutput()->setProperty( 'bs-page-access', $args['groups'] );
		return '';
	}

	public function onBSInsertMagicAjaxGetData( &$oResponse, $type ) {
		if( $type != 'tags' ) return true;

		$oResponse->result[] = array(
			'id'   => 'bs:pageaccess',
			'type' => 'tag',
			'name' => 'pageaccess',
			'desc' => wfMessage( 'bs-pageaccess-tag-groups-desc' )->plain(),
			'code' => '<bs:pageaccess groups="" />',
		);
		
		return true;
	}
}
