<?php
/**
 * SaferEdit extension for BlueSpice
 *
 * Intermediate saving of wiki edits.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    1.20.1
 * @version    $Id: SaferEdit.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage SaferEdit
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/* Changelog
 * v1.20.0
 * - MediaWiki I18N
 * v1.1.0
 * - Added indexes to table for improved performance
 * v1.0.0
 * - Raised to stable, due to EXTINFO::STATUS
 * - Code Review
 * v0.1
 * - initial commit
 */

/**
 * Base class for SaferEdit extension
 * @package BlueSpice_Extensions
 * @subpackage SaferEdit
 */
class SaferEdit extends BsExtensionMW {

	/**
	 * Constructor of PageTemplates class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['SaferEdit'] = dirname( __FILE__ ) . '/SaferEdit.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::VARIABLE;
		$this->mInfo = array(
			EXTINFO::NAME        => 'SaferEdit',
			EXTINFO::DESCRIPTION => 'Intermediate saving of wiki edits.',
			EXTINFO::AUTHOR      => 'Markus Glaser',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array(
										'bluespice'   => '1.21.0',
										'MW::StateBar' => '1.20.0'
										)
		);
		$this->mExtensionKey = 'MW::SaferEdit';

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Initialization of SaferEdit extension
	 */
	protected function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );

		BsConfig::registerVar( 'MW::SaferEdit::UseSE', false, BsConfig::LEVEL_USER|BsConfig::TYPE_BOOL|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-UseSE', 'toggle' );
		BsConfig::registerVar( 'MW::SaferEdit::HasTexts', false, BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_BOOL|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-HasTexts', 'toggle' );
		BsConfig::registerVar( 'MW::SaferEdit::EditSection', -1, BsConfig::LEVEL_PRIVATE|BsConfig::TYPE_INT|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-EditSection', 'int' );
		BsConfig::registerVar( 'MW::SaferEdit::Interval', 30, BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_INT|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-Interval', 'int' );
		BsConfig::registerVar( 'MW::SaferEdit::ShowNameOfEditingUser', true, BsConfig::LEVEL_PUBLIC|BsConfig::TYPE_BOOL|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-ShowNameOfEditingUser', 'toggle' );
		BsConfig::registerVar( 'MW::SaferEdit::WarnOnLeave', true, BsConfig::LEVEL_USER|BsConfig::TYPE_BOOL|BsConfig::RENDER_AS_JAVASCRIPT, 'bs-saferedit-pref-WarnOnLeave', 'toggle' );

		$this->setHook( 'LoadExtensionSchemaUpdates' );

		/*if ( BsConfig::get( 'MW::SaferEdit::UseSE' ) === false ) {
			wfProfileOut( 'BS::'.__METHOD__ );
			return;
		}*/

		$this->setHook( 'ArticleSaveComplete', 'clearSaferEdit' );
		$this->setHook( 'SkinTemplateOutputPageBeforeExec', 'parseSaferEdit' );
		$this->setHook( 'EditPage::showEditForm:initial', 'setEditSection' );
		$this->setHook( 'BSStateBarAddSortTopVars', 'onStatebarAddSortTopVars' );
		$this->setHook( 'BeforeInitialize' );

		BsCore::getInstance( 'MW' )->getAdapter()->addRemoteHandler( 'SaferEdit', $this, 'doSaveText', 'edit' );
		BsCore::getInstance( 'MW' )->getAdapter()->addRemoteHandler( 'SaferEdit', $this, 'doCancelSaferEdit', 'edit' );
		BsCore::getInstance( 'MW' )->getAdapter()->addRemoteHandler( 'SaferEdit', $this, 'getLostTexts', 'edit' );

		$this->registerScriptFiles( BsConfig::get( 'MW::ScriptPath' ).'/bluespice-mw/ext/SaferEdit/js', 'saferedit', true, true, false, 'MW::SaferEdit' );

		$this->mAdapter->registerBehaviorSwitch( 'NOSAFEREDIT', array( $this, 'noSaferEditCallback' ) ) ;

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Sets up required database tables
	 * @param DatabaseUpdater $updater Provided by MediaWikis update.php
	 * @return boolean Always true to keep the hook running
	 */
	public function onLoadExtensionSchemaUpdates( $updater ) {
		global $wgDBtype, $wgExtNewTables, $wgExtNewIndexes;
		$sDir = dirname( __FILE__ ) . DS;

		if( $wgDBtype == 'mysql' ) {
			$wgExtNewTables[]  = array( 'bs_saferedit', $sDir . 'SaferEdit.sql' );
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_title',     $sDir . 'db/mysql/SaferEdit.patch.se_page_title.index.sql' );
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_namespace', $sDir . 'db/mysql/SaferEdit.patch.se_page_namespace.index.sql' );
			
		} elseif( $wgDBtype == 'postgres' ) {
			$wgExtNewTables[]  = array( 'bs_saferedit', $sDir . 'SaferEdit.pg.sql' );
			/*
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_title',     $sDir . 'db/postgres/SaferEdit.patch.se_page_title.index.pg.sql' );
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_namespace', $sDir . 'db/postgres/SaferEdit.patch.se_page_namespace.index.pg.sql' );
			*/
		} elseif( $wgDBtype == 'oracle' ) {
			$wgExtNewTables[]  = array( 'bs_saferedit', $sDir . 'SaferEdit.oci.sql' );
			/*
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_title',     $sDir . 'db/oracle/SaferEdit.patch.se_page_title.index.oci.sql' );
			$wgExtNewIndexes[] = array( 'bs_saferedit', 'se_page_namespace', $sDir . 'db/oracle/SaferEdit.patch.se_page_namespace.index.oci.sql' );
			*/
		}
		return true;
	}

	/**
	 * Callback function for NOSAFEREDIT behavior switch
	 * @return bool always true
	 */
	public function noSaferEditCallback() {
		BsExtensionManager::removeContext('MW::SaferEdit');
		BsConfig::set( 'MW::SaferEdit::Use', false );
		return true;
	}

	/**
	 * Clear all previously saved intermediate edits when article is saved
	 * Called by ArticleSaveComplete hook
	 * @param Article $article The article that is created.
	 * @param User $user User that saved the article.
	 * @param string $text New text.
	 * @param string $summary Edit summary.
	 * @param bool $minoredit Marked as minor.
	 * @param bool $watchthis Put on watchlist.
	 * @param int $sectionanchor Not in use any more.
	 * @param int $flags Bitfield.
	 * @param Revision $revision New revision object.
	 * @return bool true do let other hooked methods be executed
	 */
	public function clearSaferEdit( $article, $user, $text, $summary, $minoredit, $watchthis, $sectionanchor, $flags, $revision ) {
		$this->doClearSaferEdit( $user->mName, $article->getTitle()->getDbKey(), $article->getTitle()->getNamespace() );
		return true;
	}

	/**
	 * Hook-Handler for Hook 'BSStatebarAddSortTopVars'
	 * @param array $aSortTopVars
	 * @return boolean Always true to keep hook running
	 */
	public function onStatebarAddSortTopVars( &$aSortTopVars ) {
		$aSortTopVars['statebartopsaferedit']        = wfMsg( 'bs-saferedit-statebartopsaferedit' );
		$aSortTopVars['statebartopsafereditediting'] = wfMsg( 'bs-saferedit-statebartopsafereditediting' );
		return true;
	}

	/**
	 * Checks if there are any saved intermediate edits and displays info as applicable. Callback for SkinTemplateOutputPageBeforeExec hook
	 * @param SkinTemplate $oSkinTemplate MediaWiki SkinTemplate object
	 * @param QuickTemplate $tpl Template fille with data that is about to be rendered
	 * @return bool true do let other hooked methods be executed
	 */
	function parseSaferEdit( &$oSkinTemplate, &$tpl ) {
		$db = wfGetDB( DB_SLAVE );

		$oTitle = Title::newFromID( $tpl->data['articleid'] );
		if ( is_null( $oTitle ) ) {
			return true;
		}

		$res = $db->select(
				'bs_saferedit',
				'*',
				array(
						"se_page_title" => $oTitle->getPrefixedDBkey(),
						"se_page_namespace" => $oTitle->getNamespace()
				),
				null,
				array( "ORDER BY" => "se_id DESC" )
		);

		while ( $row = $db->fetchObject( $res ) ) {
			$oArticle = Article::newFromID( $oTitle->getArticleID() );
			if ( is_object( $oArticle ) ) {
				$sOrigText = $oArticle->getRawText();
				if ( $row->se_edit_section != -1 ) {
					global $wgParser;
					$sOrigText = $wgParser->getSection( $sOrigText, $row->se_edit_section );
				}
			} else {
				$sOrigText = "";
			}
			if ( strcmp( $sOrigText, urldecode( $row->se_text ) ) == 0 ) {
				$this->doClearSaferEdit( $oSkinTemplate->username, $oTitle->getPrefixedDBkey(), $oTitle->getNamespace() );
				return true;
			}

			if ( $row->se_user_name == $oSkinTemplate->username ) {
				if ( BsExtensionMW::getInstanceFor( 'MW::StateBar' ) && BsConfig::get( 'MW::SaferEdit::UseSE' ) != false ) {
					StateBar::addTopElement( $this->makeStateBarTopSaferEdit( $row->se_edit_section ), 'statebartopsaferedit' );
				}

				if ( BsAdapterMW::getAction() == 'edit' ) {
					BsConfig::set( 'MW::SaferEdit::HasTexts', true );
				}
			}
			if ( $row->se_timestamp > date( "YmdHis", time() - BsConfig::get( 'MW::SaferEdit::Interval' ) * 10 ) ) {
				if ( BsExtensionMW::getInstanceFor( 'MW::StateBar' ) && $row->se_user_name != $oSkinTemplate->username ) {
					StateBar::addTopElement( $this->makeStateBarTopSomeoneEditing( $row->se_user_name ), 'statebartopsafereditediting' );
				}
			}
		}

		$db->freeResult( $res );
		return true;
	}

	/**
	 * Checks whether the current context is a section edit. Callback function for EditPage::showEditForm:initial hook.
	 * @param EditPage $editPage
	 * @return bool true do let other hooked methods be executed
	 */
	public function setEditSection( $editPage ) {
		BsConfig::set( 'MW::SaferEdit::EditSection', BsCore::getParam( 'section', -1, BsPARAMTYPE::STRING|BsPARAM::REQUEST ) );
		return true;
	}

	// TODO RBV (19.05.11 08:54): Use XHRResponse Abstraction from Core
	/**
	 * Saves a stored intermediate text. Called as AJAX function
	 * @param string $sOutput result of the saving operation
	 * @return bool true do let other hooked methods be executed
	 */
	public function doSaveText( &$sOutput ) {
		$sText          = BsCore::getParam( 'text', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$sUsername      = BsCore::getParam( 'uname', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$sPageTitle     = BsCore::getParam( 'pageName', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$iPageNamespace = BsCore::getParam( 'nsnumber', 0, BsPARAMTYPE::INT|BsPARAM::REQUEST );
		$iSection       = BsCore::getParam( 'section', -1, BsPARAMTYPE::INT|BsPARAM::REQUEST );
		$bPingOnly      = BsCore::getParam( 'pingOnly', false, BsPARAMTYPE::BOOL|BsPARAM::REQUEST );

		$db = wfGetDB( DB_MASTER );
		if ( !$bPingOnly ) {
			$this->doClearSaferEdit( $sUsername, $sPageTitle, $iPageNamespace );

			$aInsertFields = array(
					"se_text"           => $sText,
					"se_user_name"      => $sUsername,
					"se_page_title"     => $sPageTitle,
					"se_page_namespace" => $iPageNamespace,
					"se_edit_section"   => $iSection,
					"se_timestamp"      => date("YmdHis")
			);
			$db->insert('bs_saferedit', $aInsertFields);
		} else {
			$db->update(
					'bs_saferedit',
					array( "se_timestamp" => date( "YmdHis" ) ),
					array(
							"se_user_name"      => $sUsername,
							"se_page_title"     => $sPageTitle,
							"se_page_namespace" => $iPageNamespace,
							"se_edit_section"   => $iSection
					)
			);
		}

		$sOutput = "OK";
		return true;
	}

	// TODO RBV (19.05.11 08:56): Use XHRResponse
	/**
	 * User decided not to use saved texts, so they are dismissed. Called as AJAX function
	 * @return bool true do let other hooked methods be executed
	 */
	public function doCancelSaferEdit( &$sOutput ) {
		$sUserName      = BsCore::getParam( 'uname', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$sPageTitle     = BsCore::getParam( 'pageName', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$iPageNamespace = BsCore::getParam( 'nsnumber', 0, BsPARAMTYPE::INT|BsPARAM::REQUEST );


		if ( $this->doClearSaferEdit( $sUserName, $sPageTitle, $iPageNamespace ) ) {
			$sOutput = 'OK';
		} else {
			$sOutput = 'ERR';
		}
		return true;
	}

	/**
	 * Actually delete all stored intermediate texts for a given user and page
	 * @param string $sUserName username of the user that edited a page
	 * @param string $sPageTitle title of the page
	 * @param int $iPageNamespace number of the namespace
	 * @return bool true do let other hooked methods be executed
	 */
	protected function doClearSaferEdit( $sUserName, $sPageTitle, $iPageNamespace ) {
		$sPageTitle = str_replace( ' ', '_', $sPageTitle );
		$db = wfGetDB( DB_MASTER );
		$db->delete( 'bs_saferedit',
				array(
						"se_user_name"      => $sUserName,
						"se_page_title"     => $sPageTitle,
						"se_page_namespace" => $iPageNamespace,
				)
			);
		return true;
	}

	/**
	 * Delivers an intermediately saved text. Called as AJAX function
	 * @param string $sOutput JSON encoded string with the renderd HTML and wiki text of a intermedia saving
	 * @return bool true do let other hooked methods be executed
	 */
	public function getLostTexts( &$sOutput ) {
		$sUname         = BsCore::getParam( 'uname', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$sPageTitle     = BsCore::getParam( 'pageName', '', BsPARAMTYPE::STRING|BsPARAM::REQUEST );
		$iPageNamespace = BsCore::getParam( 'nsnumber', 0, BsPARAMTYPE::INT|BsPARAM::REQUEST );
		$iSection       = BsCore::getParam( 'section', -1, BsPARAMTYPE::INT|BsPARAM::REQUEST );

		$oDbw = wfGetDB( DB_SLAVE );

		$res = $oDbw->select(
				'bs_saferedit',
				'se_text, se_timestamp, se_edit_section',
				array(
						"se_page_title"     => $sPageTitle,
						"se_page_namespace" => $iPageNamespace,
						"se_user_name"      => $sUname
				),
				'',
				array( "ORDER BY" => "se_id DESC" )
		);
		if ( $oDbw->numRows( $res ) > 0 ) {
			$row = $oDbw->fetchRow( $res );

			$oTitle   = Title::newFromText( $sPageTitle );
			$oArticle = Article::newFromID( $oTitle->getArticleID() );
			if( is_object( $oArticle ) ) {
				$sOrigText = $oArticle->getRawText();
				if ( $iSection != -1 ) {
					global $wgParser;
					$sOrigText = $wgParser->getSection( $sOrigText, $iSection );
				}
			} else {
				$sOrigText = '';
			}
			if ( $iSection != $row['se_edit_section'] ) {
				if ( $row['se_edit_section'] == '-1' ) {
					$sEditUrl = $oTitle->getEditURL();
				} else {
					$sEditUrl = $oTitle->getEditURL()."&section=".$row['se_edit_section'];
				}
				$aData = array(
						"ts" => BsFormatConverter::mwTimestampToAgeString( $row['se_timestamp'] ),
						"savedOtherSection" => "1",
						"redirect" => $sEditUrl
				);
			} else if ( strcmp( $sOrigText, urldecode($row['se_text'] ) ) == 0 ) {
				$aData = array( "notexts" => "1" );
			} else {
				$str = urldecode($row['se_text']);
				$aData = array(
					"ts" => BsFormatConverter::mwTimestampToAgeString( $row['se_timestamp'] ),
					"html" => $this->mAdapter->parseWikiText( $str ),
					"wiki" => $str,
					"section" => $row['se_edit_section'],
					"notexts" => 0
				);
			}

			$oDbw->freeResult( $res );
		} else {
			$aData = array( "notexts" => "1" );
		}

		$sOutput = json_encode( $aData ); // TODO RBV (19.05.11 09:05): XHRResponse. Or MediaWiki AjaxResponse...

		return true;
	}

	/**
	 * Renders a note that there are lost texts to the statebar
	 * @param integer $iEditSection if lost text is from a section edit, number of the edited section
	 * @return ViewStateBarTopElement View that is to be displayed in StateBar Top
	 */
	public function makeStateBarTopSaferEdit( $iEditSection = -1 ) {
		$oSaferEditView = new ViewStateBarTopElement();
		$oArticle = BsCore::getInstance( 'MW' )->getAdapter()->get( 'Article' );
		if ( is_object( $oArticle ) ) {
			$sArticleEditPageLink = $oArticle->getTitle()->getEditURL();

			if ( $iEditSection != -1 ) {
				$sArticleEditPageLink .= '&section='.$iEditSection;
			}

			$oSaferEditView->setKey( 'SaferEdit' );
			// TODO RBV (19.05.11 09:07): Why use image from Adapter?
			$oSaferEditView->setIconSrc( $this->mAdapter->ScriptPath.'/bluespice-mw/adapter/images/bs-saferedit.png' );
			$oSaferEditView->setIconAlt( wfMsg( 'bs-saferedit-safer-edit-tooltip' ) );
			$oSaferEditView->setText( wfMsg( 'bs-saferedit-safer-edit-topbar' ) );
			$oSaferEditView->setTextLink( $sArticleEditPageLink );
			$oSaferEditView->setTextLinkTitle( wfMsg( 'bs-saferedit-safer-edit-tooltip' ) );
		}
		return $oSaferEditView;
	}

	// TODO MRG (04.05.11 01:09): Consider case where more than one editors are editing the page
	/**
	 * Renders a note that someone is editing a page to the statebar
	 * @param string $sUserName name of the user that is editing the page
	 * @return ViewStateBarTopElement View that is to be displayed in StateBar Top
	 */
	public function makeStateBarTopSomeoneEditing( $sUserName ) {
		$oSaferEditView = new ViewStateBarTopElement();

		$oSaferEditView->setKey( 'SaferEditSomeoneEditing' );
		// TODO MRG (11.01.11 14:39): use different image
		$oSaferEditView->setIconSrc( $this->mAdapter->ScriptPath.'/bluespice-mw/adapter/images/bs-infobar-editing-orange.png' );
		if ( BsConfig::get( 'MW::SaferEdit::ShowNameOfEditingUser' ) ) {
			$oSaferEditView->setIconAlt( wfMsg( 'bs-saferedit-user-editing-tooltip', $sUserName ) );
			$oSaferEditView->setText( wfMsg( 'bs-saferedit-user-editing-topbar', $sUserName ) );
		} else {
			$oSaferEditView->setIconAlt( wfMsg( 'bs-saferedit-someone-editing-tooltip' ) );
			$oSaferEditView->setText( wfMsg( 'bs-saferedit-someone-editing-topbar' ) );
		}
		//$oSaferEditView->setTextLink( $sArticleEditPageLink );
		//$oSaferEditView->setTextLinkTitle( wfMsg( 'safer-edit-tooltip' ) );

		return $oSaferEditView;
	}

	/**
	 * ParserFirstCallInit Hook is called when the parser initialises for the first time.
	 * @param Parser $parser MediaWiki Parser object
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onBeforeInitialize( &$title, &$article, &$output, &$user, $request, $mediaWiki ) {
		if( $this->checkContext($title) === false ) return true;
		BsExtensionManager::setContext( 'MW::SaferEdit' );
		return true;
	}

	/**
	 * Checks wether to set Context or not.
	 * @param OutputPage $oOutputPage
	 * @param Skin $oSkin
	 * @return bool
	 */
	private function checkContext( $oTitle ) {
		if( !is_object( $oTitle ) )							return false;
		if ( $this->mAdapter->getAction() !== 'edit' 
			&& $this->mAdapter->getAction() !== 'submit' )	return false;
		if( !$oTitle->userCan('edit'))						return false;
		if( $oTitle->getNamespace()    === NS_SPECIAL )		return false;
		return true;
	}
}
