/**
 * SaferEdit extension
 *
 * Part of BlueSpice for MediaWiki
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    $Id: saferedit.js 8506 2013-02-07 16:11:31Z smuggli $
 * @package    Bluespice_Extensions
 * @subpackage SaferEdit
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */

/**
 * Base class for all safer edit related methods and properties
 */
BsSaferEdit = {
	/**
	 * Internationalisation
	 * @var object list of i18n key value pairs
	 */	
	i18n: {
		lastSavedVersion: 'Last saved version: ',
		editFormOk : 'OK',
		editFormCancel : 'Cancel',
		restore : 'Restore',
		cancel : 'Cancel',
		unsavedChanges : 'You still have unsaved changes.',
		othersectiontitle: 'Other section to restore',
		othersectiontext1: 'There is a backup of an other section.',
		othersectiontext2: 'The backup was made ',
		othersectiontext3: 'Do you want to switch to that section?'
	},
	/**
	 * Time between two intermediate saves
	 * @var integer time in seconds
	 */
	interval: bsSaferEditInterval * 1000,
	/**
	 * Pointer to javascript timeout callback, needed to cancel timeout when changes are saved
	 * @var pointer javascript timeout callback
	 */
	timeout: false,
	/**
	 * Store for older text before safer edit saving in order to compare it to current text to see if any changes were made
	 * @var string text at time of page call or last safer edit saving
	 */
	oldText: '',
	/**
	 * Store for initial text before saving in order to compare it to current text to see if any changes were made
	 * @var string text at time of page call or last saving
	 */
	origText: '',
	/**
	 * Indicates if text was changed client side since the last safer edit saving
	 * @var bool true if text was changed
	 */
	isDirty: false,
	/**
	 * Indicates if text was changed client side since the last user saving
	 * @var bool true if text was changed
	 */
	isUnsaved: false,
	/**
	 * Indicates if the editform is submitted
	 * @var bool
	 */
	isSubmit: false,
	/**
	 * Indicates whether page is in edit mode and saving of texts should be started
	 * @var bool true if page is in edit mode
	 */
	editMode: false,
	/**
	 * Window object that stores the SaferEdit dialogue
	 * @var Ext.Window Instance of a window object
	 */
	win: false,
	/**
	 * Timestamp of saved version in restore dialogue
	 * @var string Rendered timestamp, currently age of stored release
	 */
	savedTS: '',
	/**
	 * Rendered HTML of saved version that is displayed in restore dialogue
	 * @var string Rendered HTML
	 */
	savedHTML: '',
	/**
	 * Wiki code of saved version that is inserted on OK in restore dialogue
	 * @var string Wiki code
	 */
	savedWikiCode: '',
	/**
	 * URL of section edit of page that is used if page is called in edit mode, but a saved part of a section is present
	 * @var string Redirect url
	 */
	redirect: '',
	/**
	 * Boolean if a backup is created
	 * @var bool backup created
	 */
	bBackupCreated: false,
	/**
	 * Boolean if oldtext should be reseted
	 * @var bool backup oldtext should be reseted
	 */
	bResetOldText: true,
	/**
	 * Used to display the SaferEdit restore dialogue. Calls getLostTexts with some delay to fix a sttrange IE bug.
	 */
	toggleDialog: function() {
		/* this is to fix a strange IE bug */
		setTimeout('BsSaferEdit.getLostTexts()', 10);
	},
	/**
	 * Renders SaferEdit restore dialogue. Data (HTML and WikiCode) is expected to be loaded already
	 */
	show: function() {
		BsSaferEdit.win = new Ext.Window({
			id: 'winSaferEdit',
			width:600,
			title:'SaferEdit',
			closeAction: 'close',
			items: [{
				xtype: "container",
				items: [{
					xtype: 'toolbar',
					autoHeight: true,
					items: [
						{
							xtype: 'tbtext',
							text: BsSaferEdit.i18n.lastSavedVersion + BsSaferEdit.savedTS
						}
					]
				}, {
					xtype: "container",
					html: BsSaferEdit.savedHTML,
					height: 400,
					style: 'background-color: #FFFFFF;padding:5px;',
					autoScroll: true
				}]
			}],
			buttons: [{
				id: 'ok-btn',
				text: BsSaferEdit.i18n.restore,
				handler: function(e) {
					BsSaferEdit.restore();
					BsSaferEdit.hide();
				},
				scope: this
			},{
				text: BsSaferEdit.i18n.cancel,
				handler: function(){
					BsSaferEdit.cancelSaferEdit();
					BsSaferEdit.canceledByUser = false;
					BsSaferEdit.hide();
				},
				scope: this
			}],
			layout: 'fit'
		});
		BsSaferEdit.win.show();
	},
	/**
	 * Hides the SaferEdit restore dialogue
	 */
	hide: function() {
		BsSaferEdit.startSaving();
		BsSaferEdit.win.close();
		// MRG (25.02.11 01:28): Dependency to StateBar
		if ( document.getElementById('sb-SaferEdit') ) {
			document.getElementById('sb-SaferEdit').style.display = 'none';
		}
	},
	/**
	 * Initiates saving of edited text in certain intervals
	 */
	startSaving: function() {
		BsSaferEdit.oldText = document.getElementById('wpTextbox1').value;
		BsSaferEdit.origText = document.getElementById('wpTextbox1').value;
		BsSaferEdit.timeout = setTimeout("BsSaferEdit.saveText()", BsSaferEdit.interval);
	},
	/**
	 * Renders the redirection dialogue if a whole page is edited but a saved text for a section is available
	 */
	showRedirect: function() {
		// Show a dialog using config options:
		Ext.Msg.show({
			title: BsSaferEdit.i18n.othersectiontitle,
			msg: BsSaferEdit.i18n.othersectiontext1 + '<br/>' + BsSaferEdit.i18n.othersectiontext2 + BsSaferEdit.savedTS + '<br />' + BsSaferEdit.i18n.othersectiontext3,
			buttons: Ext.Msg.YESNO,
			fn: BsSaferEdit.processRedirect,
			animEl: 'elId',
			icon: Ext.MessageBox.QUESTION
		});
	},
	/**
	 * Callback function called in showRedirect that redirects to target page
	 */
	processRedirect: function(buttonId, text, opt) {
		if ( buttonId == 'yes' ) {
			window.location.href = BsSaferEdit.redirect;
		} else {
			BsSaferEdit.cancelSaferEdit();
			BsSaferEdit.startSaving();
		}
	},
	/**
	 * Retrieves a saved intermediate text if present
	 */
	getLostTexts: function() {
		if ( typeof( bsSaferEditUseSE ) != "undefined" && bsSaferEditUseSE ) {
			var url = BlueSpice.buildRemoteString(
				'SaferEdit',
				'getLostTexts',
				{
					"pageName": wgPageName,
					"nsnumber": wgNamespaceNumber,
					"uname":wgUserName,
					"section": bsSaferEditEditSection
				}
			);
			$.get(
				url,
				null,
				function ( sResponseData ){
					var oResponse = eval( '(' + sResponseData + ')' ); // TODO RBV (19.05.11 09:33): Abstraktion verwenden?

					if ( oResponse.notexts == "1" ) return;
					if ( oResponse.savedOtherSection == "1" ) {
						BsSaferEdit.savedTS = oResponse.ts;
						BsSaferEdit.redirect = oResponse.redirect;
						BsSaferEdit.showRedirect();
						return;
					}
					BsSaferEdit.savedTS = oResponse.ts;
					BsSaferEdit.savedHTML = unescape(oResponse.html);
					BsSaferEdit.savedWikiCode = unescape(oResponse.wiki);

					setTimeout('BsSaferEdit.show()', 10);
				}
			);
		} else {
			BsSaferEdit.timeout = setTimeout("BsSaferEdit.saveText()", BsSaferEdit.interval);
		}
	},
	/**
	 * Conducts neccessary preparations of edit form and starts intermediate saving
	 */
	init: function() {
		if ( wgAction=="edit" || wgAction=="submit" ) BsSaferEdit.editMode = true;
		if ( wgCanonicalNamespace=="Special" ) BsSaferEdit.editMode = false;

		if ( BsSaferEdit.editMode ) {
			var links = document.getElementsByTagName("a");
			for ( i = 0; i < links.length; i++ ) {
				if ( links[i].innerHTML == BsSaferEdit.i18n.cancel || links[i].innerHTML == "Cancel" ) {
					links[i].onclick = BsSaferEdit.cancelSaferEdit;
				}
			}
			document.getElementById('editform').onsubmit=BsSaferEdit.cancelSaferEdit;
			//btnSave.onclick="alert('test');";
		}
		if ( BsSaferEdit.editMode && !bsSaferEditHasTexts ) {
			BsSaferEdit.startSaving();
		}
	},
	/**
	 * Retrieves edited text from textfield or editor and sends it to server
	 */
	saveText: function() {
		var text = "";
		if ( VisualEditorMode ) {
			if ( BsSaferEdit.bResetOldText === true ) {
				BsSaferEdit.oldText = tinyMCE.activeEditor.getContent({save:true});
				BsSaferEdit.bResetOldText = false;
			}
			text = tinyMCE.activeEditor.getContent({save:true});
		} else {
			text = document.getElementById('wpTextbox1').value;
		}
		
		if( BsSaferEdit.canceledByUser ) return;

		if (BsSaferEdit.oldText != text) {
			BsSaferEdit.isDirty = true;
			BsSaferEdit.oldText = text;
			BsSaferEdit.sendText(text);
			BsSaferEdit.bBackupCreated = true;
		} else {
			if ( BsSaferEdit.bBackupCreated === true ) {
				BsSaferEdit.sendText( false );
			}
			else {
				BsSaferEdit.timeout = setTimeout("BsSaferEdit.saveText()", BsSaferEdit.interval);
			}
		}
	},
	/**
	 * Sends a changed text to the server.
	 * @param string text the text that sould be sent to server
	 */
	sendText: function(text) {
		var bPingOnly = false;
		if ( text === false ) bPingOnly = true;
		$.post(
			BlueSpice.buildRemoteString('SaferEdit', 'doSaveText'),
			$.param({
				text: encodeURIComponent(text),
				uname: wgUserName,
				pageName: wgPageName,
				nsnumber: wgNamespaceNumber,
				section: bsSaferEditEditSection,
				pingOnly : bPingOnly
			}),
			function ( sResponseData ){
				if ( sResponseData == "OK" ) {
					//setTimeout('document.getElementById("hw_se_icon").style.display="none"', 200);
					BsSaferEdit.timeout = setTimeout("BsSaferEdit.saveText()", BsSaferEdit.interval); // TODO RBV (19.05.11 09:41): XHRResponse Abstraktion?
				}
			}
		);
	},
	/**
	 * Resets "dirty"-state if all changes are saved.
	 */
	clearDirty: function() {
		BsSaferEdit.isDirty = false;
	},
	// flag gesetzt um zu verhindern, dass doSaferEdit nach submit ausgeführt wird
	canceledByUser: false,
	/**
	 * All saved texts for the current article are deleted.
	 */
	cancelSaferEdit: function() {
		// flag gesetzt um zu verhindern, dass doSaferEdit nach submit ausgeführt wird
		BsSaferEdit.canceledByUser = true;
		BsSaferEdit.clearDirty();

		$.post(
			BlueSpice.buildRemoteString('SaferEdit', 'doCancelSaferEdit'),
			$.param({
				uname: escape(wgUserName),
				pageName: wgPageName,
				nsnumber: wgNamespaceNumber
			}),
			function ( sResponseData ){
				//BsSaferEdit.clearIcon();
				// TODO RBV (19.05.11 09:42): Implement
			}
		);
	},
	/**
	 * Writes text that is to be restored into the text field or editor
	 */
	restore: function() {
		var text = BsSaferEdit.savedWikiCode;
		//text = text.replace(/^<textarea.*?>/i, '');
		//text = text.replace(/<\/textarea>$/i, '');
		if ( bsVisualEditorUse ) {
			tinyMCE.execCommand('mceSetContent', false, text);
		} else {
			document.getElementById('wpTextbox1').value = text;
		}
	},

	hasUnsavedChanges: function(mode) {
		var text = "";
		if (mode=="VisualEditor") {
			text = document.getElementById('wpTextbox1').value;
		} else if (mode=="MW") {
			text = tinyMCE.activeEditor.getContent({save:true});
		} else {
			if ( VisualEditorMode ) {
				text = tinyMCE.activeEditor.getContent({save:true});
			} else {
				text = document.getElementById('wpTextbox1').value;
			}
		}
		if ( text.trim() != BsSaferEdit.origText.trim() ) {
			BsSaferEdit.isUnsaved = true;
			return true;
		} else {
			BsSaferEdit.isUnsaved = false;
			return false;
		}
	},
	/**
	 * Called when edit page is left and there are unsaved changes.
	 */
	checkSaved: function() {
		if ( !BsSaferEdit.isSubmit && BsSaferEdit.hasUnsavedChanges("-") ) {
			if(/chrome/.test(navigator.userAgent.toLowerCase())) { //chrome compatibility
				return BsSaferEdit.i18n.unsavedChanges;
			}
			if(window.event) {
				window.event.returnValue = BsSaferEdit.i18n.unsavedChanges;
			}
			else {
				return BsSaferEdit.i18n.unsavedChanges;
			}
		}
		// do not return anything, not even null. otherwise IE will display the dialogue
	},
	onToggleEditor: function(name, data) {
		if ( !BsSaferEdit.isUnsaved ) {
			var text = "";
			if ( data=="VisualEditor" ) {
				text = document.getElementById('wpTextbox1').value;
			} else if (data=="MW") {
				text = tinyMCE.activeEditor.getContent({save:true});
			} else {
				if ( VisualEditorMode ) {
					text = tinyMCE.activeEditor.getContent({save:true});
				} else {
					text = document.getElementById('wpTextbox1').value;
				}
			}
			BsSaferEdit.origText = text;
		}
	},
	onBeforeToggleEditor: function(name, data) {
		BsSaferEdit.hasUnsavedChanges(data);
	},
	onSavedText: function(name) {
		BsSaferEdit.origText = tinyMCE.activeEditor.getContent({save:true});
		BsSaferEdit.isUnsaved = false;
	}
}

$(document).ready( BsSaferEdit.init );
	if (bsSaferEditHasTexts) $(document).ready( BsSaferEdit.toggleDialog );
	if (bsSaferEditWarnOnLeave && (typeof(alreadyBound) == 'undefined' || alreadyBound == false)) {
		$(window).bind( 'beforeunload', function() {
			alreadyBound = true;
			// if a string is returned, a dialog is displayed.
			// if null is returned, nothing happenes and the page is left.
			return BsSaferEdit.checkSaved();
		});
	
	$('#editform').submit( function(){
		BsSaferEdit.isSubmit = true;
	});
}

$(document).bind('BSVisualEditorBeforeToggleEditor', BsSaferEdit.onBeforeToggleEditor );
$(document).bind('BSVisualEditorToggleEditor', BsSaferEdit.onToggleEditor );
$(document).bind('BSVisualEditorSavedText', BsSaferEdit.onSavedText );