<?php

/**
 * WhoIsOnline extension for BlueSpice
 *
 * Displays a list of users who are currently online.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * This file is part of BlueSpice for MediaWiki
 * For further information visit http://www.blue-spice.org
 *
 * @author     Markus Glaser <glaser@hallowelt.biz>
 * @version    1.20.0
 * @version    $Id: WhoIsOnline.class.php 9690 2013-06-12 10:13:02Z pwirth $
 * @package    BlueSpice_Extensions
 * @subpackage WhoIsOnline
 * @copyright  Copyright (C) 2011 Hallo Welt! - Medienwerkstatt GmbH, All rights reserved.
 * @license    http://www.gnu.org/copyleft/gpl.html GNU Public License v2 or later
 * @filesource
 */
/* Changelog
 * v1.20.0
 * - Added hook handler for InsertMagic
 * v1.1.0
 * - Added indexes to table to improve performance
 * - Added profile statements
 * - Reduced number of logging actions
 * v1.0.0
 * - Raised to stable
 * - Implemented Widget Keyword
 * v0.6
 * - Applied new coding conventions
 * - Logs SpecialPages too
 * v0.5
 * - introduced views
 * - improved error handling
 * v0.4
 * - fixed bug that if whoisonline-table not existant bluespice breaks
 * v0.3
 * - reactivated tooltip
 * - added widget hook
 * v0.2.0b
 * - Refactored / beautified code
 * - Changed SELECT querys, so they return only datasets which are not older than 10 Minutes
 * - Added new configuration variable MW::WhoIsOnline::MaxIdleTime, which
 *   tells the extenstion how long a user is allowed to be idle before being
 *   removed from the 'who's online' list.
 * - Table cleanup after 30 days ( = 720 hours )
 * - New database table scheme
 * - TO DO: Discuss wether a a minimalistic database scheme (id, user_id, page_id, timesamp) would be sufficient.
 * v0.1
 * - initial release
 */

/**
 * Base class for WhoIsOnline extension
 * @package BlueSpice_Extensions
 * @subpackage WhoIsOnline
 */
class WhoIsOnline extends BsExtensionMW {

	/**
	 * Constructor of MailChanges class
	 */
	public function __construct() {
		wfProfileIn( 'BS::'.__METHOD__ );
		global $wgExtensionMessagesFiles;
		$wgExtensionMessagesFiles['WhoIsOnline'] = dirname( __FILE__ ) . '/WhoIsOnline.i18n.php';

		// Base settings
		$this->mExtensionFile = __FILE__;
		$this->mExtensionType = EXTTYPE::PARSERHOOK;
		$this->mInfo = array(
			EXTINFO::NAME        => 'WhoIsOnline',
			EXTINFO::DESCRIPTION => 'Displays a list of users who are currently online.',
			EXTINFO::AUTHOR      => 'Markus Glaser',
			EXTINFO::VERSION     => '1.21.0 ($Rev: 9690 $)',
			EXTINFO::STATUS      => 'stable',
			EXTINFO::URL         => 'http://www.hallowelt.biz',
			EXTINFO::DEPS        => array(
				'bluespice' => '1.21.0'
			)
		);

		$this->mExtensionKey = 'MW::WhoIsOnline';

		$this->registerView( 'ViewWhoIsOnlineTag' );
		$this->registerView( 'ViewWhoIsOnlineItemWidget' );
		$this->registerView( 'ViewWhoIsOnlineWidget' );

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Initialization of ShoutBox extension
	 */
	protected function initExt() {
		wfProfileIn( 'BS::'.__METHOD__ );

		// Hooks
		$this->setHook( 'ParserFirstCallInit' );
		$this->setHook( 'LanguageGetMagic' );
		$this->setHook( 'ParserBeforeStrip' );
		$this->setHook( 'LoadExtensionSchemaUpdates' );
		$this->setHook( 'BSWidgetBarGetDefaultWidgets' );
		$this->setHook( 'BSWidgetListHelperInitKeyWords' );
		$this->setHook( 'BSInsertMagicAjaxGetData', 'onBSInsertMagicAjaxGetData' );

		//BsEventDispatcher::getInstance( 'MW' )->connect( 'MW::WidgetBar::DefaultWidgets', array( $this, 'onWidgetBarDefaultWidgets' ) );
		//BsEventDispatcher::getInstance( 'MW' )->connect( 'MW::Utility::WidgetListHelper::InitKeywords', array( $this, 'onWidgetListInitKeywords' ) );

		BsConfig::registerVar( 'MW::WhoIsOnline::LinkUsers', true, BsConfig::LEVEL_PUBLIC | BsConfig::TYPE_BOOL, 'bs-whoisonline-pref-LinkUsers', 'toggle' );
		BsConfig::registerVar( 'MW::WhoIsOnline::LimitCount', 10, BsConfig::LEVEL_USER | BsConfig::TYPE_INT, 'bs-whoisonline-pref-LimitCount', 'int' );
		BsConfig::registerVar( 'MW::WhoIsOnline::OrderBy', 'name', BsConfig::LEVEL_USER | BsConfig::TYPE_STRING | BsConfig::USE_PLUGIN_FOR_PREFS, 'bs-whoisonline-pref-OrderBy', 'select' );
		BsConfig::registerVar( 'MW::WhoIsOnline::MaxIdleTime', 600, BsConfig::LEVEL_PUBLIC | BsConfig::TYPE_INT, 'bs-whoisonline-pref-MaxIdleTime', 'int' );
		BsConfig::registerVar( 'MW::WhoIsOnline::RemoveEntriesAfter', 2592000, BsConfig::LEVEL_PRIVATE | BsConfig::TYPE_INT );
		
		$this->mAdapter->addRemoteHandler( 'WhoIsOnline', $this, 'getRemotePortlet', 'read' );

		wfProfileOut( 'BS::'.__METHOD__ );
	}

	/**
	 * Sets up required database tables
	 * @param DatabaseUpdater $updater Provided by MediaWikis update.php
	 * @return boolean Always true to keep the hook running
	 */
	public function onLoadExtensionSchemaUpdates( $updater ) {
		global $wgDBtype, $wgExtNewTables, $wgExtModifiedFields, $wgExtNewIndexes, $wgExtNewFields;
		$sDir = dirname( __FILE__ ) . DS;

		if( $wgDBtype == 'mysql' ) {
			$wgExtNewTables[] = array( 'bs_whoisonline', $sDir . 'whoisonline.sql' );

			$wgExtModifiedFields[] = array( 'bs_whoisonline', 'wo_timestamp', $sDir . 'db/mysql/whoisonline.patch.wo_timestamp.sql' );

			$wgExtNewFields[] = array( 'bs_whoisonline', 'wo_action', $sDir . 'db/mysql/whoisonline.patch.wo_action.sql' );

			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_user_id',        $sDir . 'db/mysql/whoisonline.patch.wo_user_id.index.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_page_namespace', $sDir . 'db/mysql/whoisonline.patch.wo_page_namespace.index.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_timestamp',      $sDir . 'db/mysql/whoisonline.patch.wo_timestamp.index.sql' );

		} elseif( $wgDBtype == 'postgres' ) {

			$wgExtNewTables[] = array( 'bs_whoisonline', $sDir . 'whoisonline.pg.sql' );

			$wgExtModifiedFields[] = array( 'bs_whoisonline', 'wo_timestamp', $sDir . 'db/postgres/whoisonline.patch.wo_timestamp.pg.sql' );
			//PW(25.06.2012): wont work on mw 1.16.5
			//global $wgExtPGNewFields;
			//$wgExtPGNewFields[] = array( 'bs_whoisonline', 'wo_action', $sDir . 'db/postgres/whoisonline.patch.wo_action.sql' );
			$dbr = wfGetDB( DB_MASTER );
			if( $dbr->tableExists( 'bs_whoisonline' ) && !$dbr->fieldExists( 'bs_whoisonline', 'wo_action' ) ) {
				$dbr->query( "ALTER TABLE ".$dbr->tableName( "bs_whoisonline" )." ADD wo_action text NOT NULL DEFAULT 'view';" );
			}

			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_user_id',        $sDir . 'db/postgres/whoisonline.patch.wo_user_id.index.pg.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_page_namespace', $sDir . 'db/postgres/whoisonline.patch.wo_page_namespace.index.pg.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_timestamp',      $sDir . 'db/postgres/whoisonline.patch.wo_timestamp.index.pg.sql' );

		} elseif( $wgDBtype == 'oracle' ) {
			$wgExtNewTables[] = array( 'bs_whoisonline', $sDir . 'whoisonline.oci.sql' );

			$wgExtModifiedFields[] = array( 'bs_whoisonline', 'wo_timestamp', $sDir . 'db/oracle/whoisonline.patch.wo_timestamp.oci.sql' );

			#$wgExtNewFields[] = array( 'bs_whoisonline', 'wo_action', $sDir . 'db/oracle/whoisonline.patch.wo_action.sql' );

			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_user_id',        $sDir . 'db/oracle/whoisonline.patch.wo_user_id.index.oci.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_page_namespace', $sDir . 'db/oracle/whoisonline.patch.wo_page_namespace.index.oci.sql' );
			$wgExtNewIndexes[] = array( 'bs_whoisonline', 'wo_timestamp',      $sDir . 'db/oracle/whoisonline.patch.wo_timestamp.index.oci.sql' );
		}
		return true;
	}
	
	public function onBSInsertMagicAjaxGetData( &$oResponse, $type ) {
		if( $type != 'tags' ) return true;

		$oResponse->result[] = array(
			'id' => 'bs:whoisonlinecount',
			'type' => 'tag',
			'name' => 'whoisonlinecount',
			'desc' => wfMsgExt( 'bs-whoisonline-tag-whoisonlinecount-desc', array( 'parse' ) ),
			'code' => '<bs:whoisonlinecount />',
		);
		
		$oResponse->result[] = array(
			'id' => 'bs:whoisonlinepopup',
			'type' => 'tag',
			'name' => 'whoisonlinepopup',
			'desc' => wfMsgExt( 'bs-whoisonline-tag-whoisonlinepopup-desc', array( 'parse' ) ),
			'code' => '<bs:whoisonlinepopup />',
		);
		
		//TODO: Add MagicWord variables too?

		return true;
	}

	/**
	 * Sets parameters for more complex options in preferences
	 * @param string $sAdapterName Name of the adapter, e.g. MW
	 * @param BsConfig $oVariable Instance of variable
	 * @return array Preferences options
	 */
	public function runPreferencePlugin( $sAdapterName, $oVariable ) {
		$aPrefs = array(
			'options' => array(
				wfMsg( 'bs-whoisonline-pref-sort-name' ) => 'name',
				wfMsg( 'bs-whoisonline-pref-sort-time' ) => 'onlinetime',
			)
		);
		return $aPrefs;
	}

	/**
	 * Event-Handler for 'MW::Utility::WidgetListHelper::InitKeywords'. Registers a callback for the WHOISONLINE Keyword.
	 * @param BsEvent $oEvent The Event object
	 * @param array $aKeywords An array of Keywords array( 'KEYWORD' => $callable )
	 * @return array The appended array of Keywords array( 'KEYWORD' => $callable )
	 */
	public function onBSWidgetListHelperInitKeyWords( &$aKeywords, $oTitle ) {
		$aKeywords[ 'WHOISONLINE' ] = array( $this, 'onWidgetListKeyword' );
		return true;
	}

	/**
	 * Callback for WidgetListHelper. Adds the WhoIsOnline Widget to the list if Keyword is found.
	 * @return ViewWidget.
	 */
	public function onWidgetListKeyword() {
		wfProfileIn( 'BS::'.__METHOD__ );
		$oOutput = json_decode( $this->getPortlet() );
		$sOut = $oOutput->html;

		$oWidgetView = new ViewWidget();
		$oWidgetView
			->setId( 'bs-whoisonline' )
			->setTitle( wfMsg( 'bs-whoisonline-widget-title' ) )
			->setBody( $sOut )
			->setTooltip( wfMsg( 'bs-whoisonline-widget-title' ) )
			->setAdditionalBodyClasses( array( 'bs-nav-links' ) ); //For correct margin and fontsize

		wfProfileOut( 'BS::'.__METHOD__ );
		return $oWidgetView;
	}

	/**
	 * Add various tags and magic words. Magic Words are only supported for legacy reasons.
	 * @param Parser $oParser Current MediaWiki Parser object
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onParserFirstCallInit( &$oParser ) {
		$oParser->setFunctionHook( 'userscount', array( &$this, 'onUsersCount' ) );
		$oParser->setHook( 'bs:whoisonline:count', array( &$this, 'onUsersCountTag' ) );
		$oParser->setHook( 'bs:whoisonlinecount', array( &$this, 'onUsersCountTag' ) );
		$oParser->setFunctionHook( 'userslink', array( &$this, 'onUsersLink' ) );
		$oParser->setHook( 'bs:whoisonline:popup', array( &$this, 'onUsersLinkTag' ) );
		$oParser->setHook( 'bs:whoisonlinepopup', array( &$this, 'onUsersLinkTag' ) );

		return true;
	}

	/**
	 * Add magic words. Used for legacy support.
	 * @param array $aMagicWords Array of magic words. Add to this array.
	 * @param string $sLangCode Current langugage.
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	public function onLanguageGetMagic( &$aMagicWords, $sLangCode ) {
		$aMagicWords[ 'userscount' ] = array( 0, 'userscount' );
		$aMagicWords[ 'userslink' ]  = array( 0, 'userslink' );
		return true;
	}

	/**
	 * Inserts a trace of the user action into the database
	 * @global WebRequest $wgRequest The MediaWiki WebRequest object
	 * @param object $oParser MediaWiki Parser Object
	 * @param string $sText The text being parsed
	 * @param object $oStripState The MediaWiki StripState object
	 * @return bool allow other hooked methods to be executed. Always true.
	 */
	function onParserBeforeStrip( $oParser, &$sText, $oStripState ) {
		global $wgRequest;

		$oCurrentUser = $this->mAdapter->get( 'User' );
		if ( ( $oCurrentUser->getId() == 0 ) ) return true; // Anonymous user

		$oCurrentTitle = $oParser->getTitle();
		$sPageTitle = $oCurrentTitle->getText();
		if ( $sPageTitle == '-' ) return true; // otherwise strange '-' with page_id 0 are logged
		$iPageId = $oCurrentTitle->getArticleId();
		$iPageNamespaceId = $oCurrentTitle->getNamespace();

		$iCurrentTimestamp = time();
		$vLastLoggedPageHash = $wgRequest->getSessionData( $this->mExtensionKey.'::lastLoggedPageHash' );
		$vLastLoggedTime     = $wgRequest->getSessionData( $this->mExtensionKey.'::lastLoggedTime' );
		$sCurrentPageHash = md5( $iPageId.$iPageNamespaceId.$sPageTitle ); //this combination should be pretty unique, even with specialpages.
		if( $vLastLoggedPageHash == $sCurrentPageHash 
			&& $vLastLoggedTime + 15 > $iCurrentTimestamp
			) return true;

		//log action
		wfProfileIn( 'BS::'.__METHOD__ );
		$wgRequest->setSessionData( $this->mExtensionKey.'::lastLoggedPageHash', $sCurrentPageHash );
		$wgRequest->setSessionData( $this->mExtensionKey.'::lastLoggedTime', $iCurrentTimestamp );

		$iRemoveEntriesAfter = BsConfig::get( 'MW::WhoIsOnline::RemoveEntriesAfter' );

		$dbw = wfGetDB( DB_MASTER );
		$dbw->delete(
			'bs_whoisonline',
			array( 'wo_timestamp < ' . ( $iCurrentTimestamp - $iRemoveEntriesAfter ) )
		);

		global $wgDBtype;
		$aNewRow = array();
		if( $wgDBtype == 'oracle' ) $aNewRow[ 'wo_id' ] = 0;
		$aNewRow[ 'wo_page_id' ]        = $oCurrentTitle->getArticleId();
		$aNewRow[ 'wo_page_namespace' ] = $oCurrentTitle->getNamespace();
		$aNewRow[ 'wo_page_title' ]     = $sPageTitle;
		$aNewRow[ 'wo_user_id' ]        = $oCurrentUser->getId();
		$aNewRow[ 'wo_user_name' ]      = $oCurrentUser->getName();
		$aNewRow[ 'wo_user_real_name' ] = $oCurrentUser->getRealName();
		$aNewRow[ 'wo_timestamp' ]      = $iCurrentTimestamp;
		$aNewRow[ 'wo_action' ]         = $wgRequest->getVal( 'action', 'view' );

		$dbw->insert( 'bs_whoisonline', $aNewRow );

		wfProfileOut( 'BS::'.__METHOD__ );
		return true;
	}

	/**
	 * Fetches the HTML for bs:whoisonline:count tag
	 * @param string $sInput Inner HTML of the tag. Not used.
	 * @param array $aAttributes List of the tag's attributes.
	 * @param Parser $oParser MediaWiki parser object.
	 * @return string Rendered HTML.
	 */
	public function onUsersCountTag( $sInput, $aAttributes, $oParser ) {
		$aOut = $this->onUsersCount( $oParser );
		return $aOut[0];
	}

	/**
	 * Renders bs:whoisonline:count output.
	 * @param Parser $oParser MediaWiki parser object.
	 * @return array Rendered HTML and flags. Used by magic word function hook as well as by onUsersCountTag.
	 */
	public function onUsersCount( $oParser ) {
		wfProfileIn( 'BS::'.__METHOD__ );
		$iCurrentTimestamp = time();

		$dbr = wfGetDB( DB_SLAVE );
		$res = $dbr->select(
				'bs_whoisonline', 
				'DISTINCT wo_user_name',
				array( 'wo_timestamp > ' . ( $iCurrentTimestamp - BsConfig::get( 'MW::WhoIsOnline::MaxIdleTime' ) ) ),
				__METHOD__,
				array( 'ORDER BY' => 'wo_user_name' )
		);

		$sOut = $dbr->numRows( $res );
		$dbr->freeResult( $res );

		wfProfileOut( 'BS::'.__METHOD__ );
		return array( $sOut, 'noparse' => 1 );
	}

	/**
	 * Fetches the HTML for bs:whoisonline:popup tag
	 * @param string $sInput Inner HTML of the tag. Not used.
	 * @param array $aAttributes List of the tag's attributes.
	 * @param Parser $oParser MediaWiki parser object.
	 * @return string Rendered HTML.
	 */
	public function onUsersLinkTag( $sInput, $aAttributes, $oParser ) {
		//Validation in onUsersLink.
		return $this->onUsersLink( $oParser, isset($aAttributes[ 'anchortext' ])?$aAttributes[ 'anchortext' ] : "" );
	}

	/**
	 * Renders bs:whoisonline:popup output.
	 * @param Parser $oParser MediaWiki parser object.
	 * @param string $sLinkTitle Label of the link that is the anchor of the flyout
	 * @return array Rendered HTML and flags. Used by magic word function hook as well as by onUsersLinkTag.
	 */
	public function onUsersLink( $oParser, $sLinkTitle = '') {
		$oParser->disableCache();
		wfProfileIn( 'BS::'.__METHOD__ );
		$sLinkTitle = BsCore::sanitize( $sLinkTitle, '', BsPARAMTYPE::STRING );
		if( empty( $sLinkTitle ) ) $sLinkTitle = wfMessage('bs-whoisonline-widget-title')->plain();
		$oWhoIsOnlineTagView = new ViewWhoIsOnlineTag();
		$oWhoIsOnlineTagView->setOption( 'title', $sLinkTitle );
		$sOut = $oWhoIsOnlineTagView->execute();

		wfProfileOut( 'BS::'.__METHOD__ );
		return $oParser->insertStripItem( $sOut, $oParser->mStripState );
	}

	/**
	 * Renders the WhoIsOnline widget
	 * @param BsEvent $oEvent
	 * @param array $aWidgets List of widgets. Add to this list.
	 * @return array Appenden list of Widgets.
	 */
	public function onBSWidgetBarGetDefaultWidgets( &$aViews, $oUser, $oTitle ) {
		$aViews[] = $this->onWidgetListKeyword();
		return true;
	}
	
	public function getRemotePortlet( &$output ) {
		$output = $this->getPortlet();
		return $output;
	}

	/**
	 * Renders the inner part of tag and widget view.
	 * @param mixed $vWrapperId Distinct ID. Used if several instances are used on a page.
	 * @return string Rendered HTML
	 */
	private function getPortlet( $vWrapperId = false ) {
		wfProfileIn( 'BS::'.__METHOD__ );

		$iLimit   = is_int(BsConfig::get( 'MW::WhoIsOnline::LimitCount' ))?BSConfig::get('MW::WhoIsOnline::LimitCount'):null;
		$sWhere   = 'wo_timestamp > ' . ( time() - BsConfig::get( 'MW::WhoIsOnline::MaxIdleTime' ) );
		$sOrderBy = BsConfig::get( 'MW::WhoIsOnline::OrderBy' );

		$dbr = wfGetDB( DB_SLAVE );
		switch ( $sOrderBy ) {
			case 'name' :
			default :
				$res = $dbr->select(
					'bs_whoisonline',
					'DISTINCT wo_user_name',
					$sWhere,
					__METHOD__,
					array( 
						'ORDER BY' => 'wo_user_name ASC',
						'LIMIT'    => $iLimit
					)
				);
			case 'onlinetime' :
				$res = $dbr->select(
						'bs_whoisonline', 
						'wo_user_name', 
						$sWhere,
						__METHOD__,
						array(
							'GROUP BY' => 'wo_user_name',
							'ORDER BY' => 'MAX(wo_timestamp) DESC',
							'LIMIT'    => $iLimit
						)
				);
		}

		// who (names)
		$oWhoIsOnlineWidgetView = new ViewWhoIsOnlineWidget();
		$oWhoIsOnlineWidgetView->setOption( 'count', $dbr->numRows( $res ) );
		$oWhoIsOnlineWidgetView->setOption( 'wrapper-id', $vWrapperId );

		while ( $row = $dbr->fetchRow( $res ) ) {
			$oUser = User::newFromName( $row[ 'wo_user_name' ] );
			$oWhoIsOnlineItemWidgetView = new ViewWhoIsOnlineItemWidget();
			$oWhoIsOnlineItemWidgetView->setOption( 'renderLink', BsConfig::get( 'MW::WhoIsOnline::LinkUsers' ) );
			$oWhoIsOnlineItemWidgetView->setUserName( $oUser->getName() );
			$oWhoIsOnlineItemWidgetView->setUserDisplayName( $this->mAdapter->getUserDisplayName( $oUser ) );
			$oWhoIsOnlineWidgetView->addItem( $oWhoIsOnlineItemWidgetView );
		}
		$aOut = array(
			'count' => $dbr->numRows( $res ),
			'html' => $oWhoIsOnlineWidgetView->execute()
		);
//		$sOut = $oWhoIsOnlineWidgetView->execute();
		$dbr->freeResult( $res );
		wfProfileOut( 'BS::'.__METHOD__ );
		return json_encode( $aOut );
//		return $sOut;
	}
}
